<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Minh <phandinhminh@protonmail.ch>
 */
$lang['menu']                  = 'Menu mục lục tiện ích';
$lang['fetch']                 = 'Hiện';
$lang['install']               = 'Cài đặt';
$lang['delete']                = 'Xoá';
$lang['check']                 = 'Kiểm tra';
$lang['no_repos']              = 'Không có url kho lưu trữ theme chủ đề nào được định cấu hình.';
$lang['disabled']              = 'Đã tắt';
$lang['conn_err']              = 'Lỗi kết nối';
$lang['dir_err']               = 'Không thể tạo thư mục tạm để nhận theme chủ đề';
$lang['down_err']              = 'Không thể nhận theme chủ đề';
$lang['zip_err']               = 'Lỗi tạo hoặc giải nén zip';
$lang['install_ok']            = 'Theme chủ đề được cài đặt thành công. Theme chủ đề mới sẵn sàng trong thanh công cụ của Chỉnh sửa trang hoặc với dòng lệnh <code>js#theme_name option</code>.';
$lang['install_no']            = 'Lỗi kết nối. Mặc dù vậy, bạn có thể thử tải lên bằng cách thủ công theme chủ đề từ <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">here</a>.';
$lang['delete_ok']             = 'Theme chủ đề đã được xoá thành công.';
$lang['delete_no']             = 'Đã có lỗi xảy ra trong khi xoá theme chủ đề';
$lang['upload']                = 'Chia sẻ';
$lang['checkupdates']          = 'Nâng cấp plugin';
$lang['noupdates']             = 'Menu mục lục không cần phải nâng cấp. Bạn đã có phiên bản mới nhất:';
$lang['infos']                 = 'Bạn có thể tạo theme chủ đề của bạn bằng cách thực hiện theo hướng dẫn tại trang <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Hướng dẫn Theme chủ đề</a>. <br />Sau đó bạn có thể khiến nhiều người vui vẻ hơn :-) bằng cách gửi chúng tới thư viện menu mục lục công cộng, với nút "chia sẻ" ngay bên dưới theme chủ đề.';
$lang['showsort']              = 'Sắp xếp số thứ tự Menu mục lục:';
$lang['donation_text']         = 'Plugin menu mục lục không được tài trợ bởi bất kỳ ai nhưng tôi phát triển và hỗ trợ nó không công trong thời gian rảnh của tôi. Nếu bạn đạt được điều gì nhờ vào nó hoặc bạn muốn hỗ trợ sự phát triển của nó, bạn có thể xem xét quyên góp.';
$lang['js']['indexmenuwizard'] = 'Trình hướng dẫn menu mục lục';
$lang['js']['index']           = 'Mục lục';
$lang['js']['options']         = 'Tuỳ chọn';
$lang['js']['navigation']      = 'Định hướng';
$lang['js']['sort']            = 'Sắp xếp';
$lang['js']['filter']          = 'Lọc';
$lang['js']['performance']     = 'Hiệu quả';
$lang['js']['namespace']       = 'Khoảng tên';
$lang['js']['nsdepth']         = 'Độ sâu';
