<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author İlker R. Kapaç <irifat@gmail.com>
 */
$lang['menu']                  = 'Indexmenu Hizmetleri';
$lang['fetch']                 = 'Göster';
$lang['install']               = 'Kur';
$lang['delete']                = 'Sil';
$lang['check']                 = 'Kontrol et';
$lang['no_repos']              = 'Hiç bir tema deposunun adresi değiştirilmedi.';
$lang['disabled']              = 'Devre dışı';
$lang['conn_err']              = 'Bağlantı hatası';
$lang['dir_err']               = 'Temayı almak için geçici klasör oluşturulamadı';
$lang['down_err']              = 'Tema alınamadı';
$lang['zip_err']               = 'Zip oluşturulurken veya açılırken hata ile karşılaşıldı';
$lang['install_ok']            = 'tema başarıyla kuruldu. Yeni tema sayfa düzenleme ekranındaki araç çubuğunda veya <code>js#theme_name option</code> ile kullanılabilir.';
$lang['install_no']            = 'Bağlantı hatası. Yine de temanızı <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">bu adresi</a> kullanarak, elle yüklemeyi deneyebilirsiniz.';
$lang['delete_ok']             = 'Tema başarıyla silindi.';
$lang['delete_no']             = 'Tema silinirken bir hatayla karşılaşıldı';
$lang['upload']                = 'Paylaş';
$lang['checkupdates']          = 'Eklenti güncellemeleri';
$lang['noupdates']             = 'Indexmenu eklentisinin güncellenmesine gerek yok. Zaten son sürümü kullanıyorsunuz.';
