<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Neli Dimitrova <neli.dimitrova.office@gmail.com>
 */
$lang['secedit_name']          = 'Таблица';
$lang['add_table']             = 'Вмъкни нова таблица';
$lang['js']['toggle_header']   = 'Промени състоянието на антетката';
$lang['js']['align_left']      = 'Подравняване на текста отляво';
$lang['js']['align_center']    = 'Центриране на текста';
$lang['js']['align_right']     = 'Подравняване на текста отдясно';
$lang['js']['confirmdeleterow'] = 'Наистина ли искате да изтриете реда?';
$lang['js']['confirmdeletecol'] = 'Наистина ли искате да изтриете колоната?';
$lang['js']['row_above']       = 'Добавяне на ред отгоре';
$lang['js']['remove_row']      = 'Премахване на ред';
$lang['js']['row_below']       = 'Добавяне на ред отдолу';
$lang['js']['col_left']        = 'Добавяне на колона отляво';
$lang['js']['remove_col']      = 'Премахване на колона';
$lang['js']['col_right']       = 'Добавяне на колона отдясно';
$lang['js']['merge_cells']     = 'Сливане на клетки';
$lang['js']['unmerge_cells']   = 'Разделяне на клетки';
