<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Domingo Redal <docxml@gmail.com>
 * @author Yllelder <yllelder@gmail.com>
 * @author Zigor Astarbe <astarbe@gmail.com>
 * @author Digna González Otero <digna.gonzalezotero [at] gmail [dot] com>
 * @author Eloy <ej.perezgomez@gmail.com>
 */
$lang['menu']                  = 'Utilidades de Indexmenu';
$lang['fetch']                 = 'Mostrar';
$lang['install']               = 'Instalar';
$lang['delete']                = 'Eliminar';
$lang['check']                 = 'Seleccionar';
$lang['no_repos']              = 'Url de repositorio de temas no configurado.';
$lang['disabled']              = 'Desactivado';
$lang['conn_err']              = 'Error de conexión';
$lang['dir_err']               = 'Imposible crear directorio temporar para almacenar el tema';
$lang['down_err']              = 'Imposible recibir el tema';
$lang['zip_err']               = 'Error de creación o extracción de Zip';
$lang['install_ok']            = 'Tema instalado satisfactoriamente. El nuevo tema está disponible en la barra de herramientas de la página a editar o con la <code>opción js#nombre_del_tema</code>.';
$lang['install_no']            = 'Error de conexión. Puedes intentar subir tu tema manualmente desde <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">here</a>.';
$lang['delete_ok']             = 'Tema eliminado correctamente.';
$lang['delete_no']             = 'Ocurrió un error durante la eliminación del tema';
$lang['upload']                = 'Compartir';
$lang['checkupdates']          = 'Actualizaciones de plugin';
$lang['noupdates']             = 'No es necesario actualizar Indexmenu. Ya tienes la última versión:';
$lang['infos']                 = 'Puedes crear tu tema siguiendo las instrucciones de la página <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a>. <br />Después puedes hacer feliz a más gente:-) enviándolo al repositorio público de indexmenu, con el botón de "compartir" bajo ese tema.';
$lang['showsort']              = 'Número de posición de Indexmenu: ';
$lang['donation_text']         = 'El plugin indexmenu no está patrocinado por nadie, pero lo desarrollo y mantengo de forma gratuita durante mi tiempo libre. Si ganas algo gracias a él o quieres apoyar su desarrollo, puedes considerar hacer una donación.';
$lang['js']['indexmenuwizard'] = 'Asistente de Indexmenu';
$lang['js']['index']           = 'Indice';
$lang['js']['options']         = 'Opciones';
$lang['js']['navigation']      = 'Navegación';
$lang['js']['sort']            = 'Ordenar';
$lang['js']['filter']          = 'Filtrar';
$lang['js']['performance']     = 'Rendimiento';
$lang['js']['namespace']       = 'Espacio de nombres';
$lang['js']['nsdepth']         = 'Profundidad';
$lang['js']['js']              = 'Árbol renderizado con Javascript, puedes definir tu propio tema';
$lang['js']['theme']           = 'Tema';
$lang['js']['navbar']          = 'El árbol se abre en el espacio de nombres actual';
$lang['js']['context']         = 'Muestra el árbol del contexto del espacio de nombres de la wiki actual';
$lang['js']['nocookie']        = 'No recordar los nodos abiertos/cerrados durante la navegación del usuario';
$lang['js']['noscroll']        = 'Evitar el desplazamiento del árbol cuando no se ajusta a la anchura del contenedor';
$lang['js']['notoc']           = 'Desactivar la función de vista previa del toc';
$lang['js']['tsort']           = 'Por título';
$lang['js']['dsort']           = 'Por fecha';
$lang['js']['msort']           = 'Por meta etiqueta';
$lang['js']['nsort']           = 'Ordenar también espacios de nombres';
$lang['js']['hsort']           = 'Ordenar página principal arriba';
$lang['js']['rsort']           = 'Invertir el orden de las páginas';
$lang['js']['nons']            = 'Mostrar solo páginas';
$lang['js']['nopg']            = 'Mostrar solo espacios de nombres';
$lang['js']['max']             = 'Cuántos niveles representar con ajax cuando se abre un nodo. Adicional son cuántos subniveles por debajo de este nivel se recuperan con AJAX en lugar de una única vez.';
$lang['js']['maxjs']           = 'Cuántos niveles se renderizan en el navegador del cliente cuando se abre un nodo';
$lang['js']['id']              = 'Cookie ID autodefinida para este indexmenu';
$lang['js']['insert']          = 'Insertar indexmenu';
$lang['js']['metanum']         = 'Meta número para ordenar';
$lang['js']['insertmetanum']   = 'Insertar metanúmero';
$lang['js']['page']            = 'Página';
$lang['js']['revs']            = 'Revisiones';
$lang['js']['tocpreview']      = 'Vista previa del toc';
$lang['js']['editmode']        = 'Modo de edición';
$lang['js']['insertdwlink']    = 'Insertar como DWlink';
$lang['js']['insertdwlinktooltip'] = 'Insertar el enlace de esta página en el cuadro de edición en la posición del cursor';
$lang['js']['ns']              = 'Espacio de nombres';
$lang['js']['search']          = 'Buscar ...';
$lang['js']['searchtooltip']   = 'Buscar páginas dentro de este espacio de nombres';
$lang['js']['create']          = 'Crear';
$lang['js']['more']            = 'Más';
$lang['js']['headpage']        = 'Página principal';
$lang['js']['headpagetooltip'] = 'Crear una nueva página principal debajo de esta página';
$lang['js']['startpage']       = 'Página de Inicio';
$lang['js']['startpagetooltip'] = 'Crea una nueva página de inicio bajo esta página';
$lang['js']['custompage']      = 'Página personalizada...';
$lang['js']['custompagetooltip'] = 'Crear una nueva página (introduce el nombre a través de una ventana emergente) bajo esta página';
$lang['js']['acls']            = 'Reglas Acls';
$lang['js']['purgecache']      = 'Purgar caché';
$lang['js']['exporthtml']      = 'Exportar como HTML';
$lang['js']['exporttext']      = 'Exportar como texto';
$lang['js']['headpagehere']    = 'Página principal aquí';
$lang['js']['headpageheretooltip'] = 'Crea una nueva página principal dentro de este espacio de nombres';
$lang['js']['newpage']         = 'Nueva página....';
$lang['js']['newpagetooltip']  = 'Cree una nueva página (ingrese el nombre mediante una ventana emergente) dentro de este espacio de nombres';
$lang['js']['newpagehere']     = 'Nueva página aquí';
$lang['js']['insertkeywords']  = 'Inserte las palabras clave para buscar dentro de este espacio de nombres';
$lang['js']['insertpagename']  = 'Inserte el nombre de la página a crear';
$lang['js']['edit']            = 'Editar';
$lang['js']['loading']         = 'Cargando...';
