<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Paul Ou <ou4222@gmail.com>
 */
$lang['fetch']                 = '顯示';
$lang['install']               = '安裝';
$lang['delete']                = '刪除';
$lang['check']                 = '檢查';
$lang['disabled']              = '關閉';
$lang['conn_err']              = '連接錯誤';
$lang['dir_err']               = '無法建立暫時資料夾來接受主題佈景';
$lang['down_err']              = '無法接受主題佈景';
$lang['zip_err']               = '壓縮檔在建立或解壓縮時發生錯誤';
$lang['delete_ok']             = '主題佈景已經順利刪除';
$lang['delete_no']             = '在刪除主題佈景時發生錯誤';
$lang['upload']                = '分享';
$lang['checkupdates']          = '插件更新';
$lang['noupdates']             = '索引選單不需要更新。你已經有最新版本：';
$lang['js']['indexmenuwizard'] = '索引選單精靈';
$lang['js']['index']           = '索引';
$lang['js']['options']         = '選項';
$lang['js']['navigation']      = '導覽';
$lang['js']['sort']            = '排序';
$lang['js']['filter']          = '過濾器';
$lang['js']['performance']     = '效率';
$lang['js']['namespace']       = '名字空間';
$lang['js']['nsdepth']         = '深度';
$lang['js']['theme']           = '主題佈景';
$lang['js']['notoc']           = '關閉內容目錄的預覽功能';
$lang['js']['tsort']           = '依照標題';
$lang['js']['dsort']           = '依照日期';
$lang['js']['msort']           = '依照後設標籤';
$lang['js']['nsort']           = '名字空間也排序';
$lang['js']['rsort']           = '將頁面排序反轉過來';
$lang['js']['nons']            = '只顯示頁面';
$lang['js']['nopg']            = '只顯示名字空間';
$lang['js']['insert']          = '插入索引選單';
