<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Bartek S <sadupl@gmail.com>
 */
$lang['menu']                  = 'Narzędzia indexmenu';
$lang['fetch']                 = 'Pokaż';
$lang['install']               = 'Instaluj';
$lang['delete']                = 'Usuń';
$lang['check']                 = 'Sprawdź';
$lang['disabled']              = 'Zablokowany';
$lang['conn_err']              = 'Błąd połączenia';
$lang['checkupdates']          = 'Aktualizacja pluginu';
$lang['no_repos']              = 'Nie skonfigurowano adresu URL repozytorium motywów.';
$lang['dir_err']               = 'Nie można utworzyć folderu tymczasowego do odebrania motywu';
$lang['down_err']              = 'Nie można odebrać motywu';
$lang['zip_err']               = 'Błąd tworzenia lub rozpakowywania zip';
$lang['install_ok']            = 'motyw zainstalowano pomyślnie. Nowy motyw jest dostępny na pasku narzędzi strony edycji lub z opcją <code>js#theme_name</code>.';
$lang['delete_ok']             = 'Motyw został skasowany pomyślnie.';
$lang['delete_no']             = 'Wystąpił błąd podczas usuwania motywu';
$lang['upload']                = 'Udostępnij';
$lang['js']['index']           = 'Indeks';
$lang['js']['options']         = 'Opcje';
$lang['js']['navigation']      = 'Nawigacja';
$lang['js']['sort']            = 'Sortuj';
$lang['js']['filter']          = 'Filtruj';
$lang['js']['tsort']           = 'Po tytule';
$lang['js']['dsort']           = 'Po dacie';
$lang['js']['msort']           = 'Po meta tag';
$lang['js']['nons']            = 'Pokaż tylko strony';
$lang['js']['page']            = 'Strona';
$lang['js']['search']          = 'Szukaj ...';
$lang['js']['performance']     = 'Wydajność';
$lang['js']['namespace']       = 'Przestrzeń';