<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author İlker R. Kapaç <irifat@gmail.com>
 */
$lang['secedit_name']          = 'Tablo';
$lang['add_table']             = 'Yeni bir tablo ekle';
$lang['js']['toggle_header']   = 'Başlık durumunu değiştir';
$lang['js']['align_left']      = 'Hücreyi sola hizala';
$lang['js']['align_center']    = 'Hücreyi ortala';
$lang['js']['align_right']     = 'Hücreyi sağa hizala';
$lang['js']['confirmdeleterow'] = 'Satır silinsin mi?';
$lang['js']['confirmdeletecol'] = 'Sütun silinsin mi?';
$lang['js']['row_above']       = 'Üste satır ekle';
$lang['js']['remove_row']      = 'Satırı sil';
$lang['js']['row_below']       = 'Alta satır ekle';
$lang['js']['col_left']        = 'Sola sütun ekle';
$lang['js']['remove_col']      = 'Sütunu sil';
$lang['js']['col_right']       = 'Sağa sütun ekle';
$lang['js']['merge_cells']     = 'Hücreleri birleştir';
$lang['js']['unmerge_cells']   = 'Hücreleri böl';
