<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * Vietnamese language file
 *
 */
$lang['namespaceRoot']         = 'Gốc';
$lang['okbutton']              = 'Tạo trang mới';
$lang['nooption']              = 'Bạn không được phép tạo trang mới';
