<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author MaWi <drmaxxis@gmail.com>
 * @author Michael <michael.kraemer@gmail.com>
 */
$lang['namespaceRoot']         = 'Wurzel';
$lang['name']                  = 'Seiten Name';
$lang['okbutton']              = 'Seite hinzufügen';
$lang['nooption']              = 'Du besitzt nicht die Benutzerrechte um Seiten hinzuzufügen.';
