<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Gerrit Uitslag <klapinklapin@gmail.com>
 * @author Joachim David <joa_david@hotmail.com>
 */
$lang['menu']                  = 'Indexmenu Themabeheerder';
$lang['fetch']                 = 'Weergeven';
$lang['install']               = 'Installeren';
$lang['delete']                = 'Verwijderen';
$lang['check']                 = 'Controleer';
$lang['no_repos']              = 'Geen thema-repositorie url ingesteld';
$lang['disabled']              = 'Uitgeschakeld';
$lang['conn_err']              = 'Verbindingsfout';
$lang['dir_err']               = 'Niet in staat een tijdelijke map te maken om thema te downloaden';
$lang['down_err']              = 'Niet in staat thema te ontvangen';
$lang['zip_err']               = 'Zip inpak- of uitpakfout';
$lang['install_ok']            = 'thema succesvol geïnstalleerd. Het nieuwe thema is beschikbaar in de werkbalk van het bewerkvenster of met <code>js#theme_name</code> syntax';
$lang['install_no']            = 'Verbindingsfout. Je kunt ook proberen om handmatig je thema te downloaden van <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">de site van de ontwikkelaar</a>.';
$lang['delete_ok']             = 'Thema is succesvol verwijderd.';
$lang['delete_no']             = 'Er is een fout gebeurd bij verwijderen thema';
$lang['upload']                = 'Delen';
$lang['checkupdates']          = 'Plugin updates';
$lang['noupdates']             = 'Indexmenu heeft geen update nodig. Je hebt al de laatste versie:';
$lang['infos']                 = 'Je kunt je eigen thema maken met de instructies op de  <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Thema handleiding</a> op de plugin wikipagina. <br /> Je kunt dan meer mensen blij maken :-) Stuur het naar de publieke indexmenu repositorie met de "Delen" knop onder dat thema.';
$lang['showsort']              = 'Indexmenu sorteernummer:';
$lang['donation_text']         = 'De indexmenu plugin wordt door niemand gesponsord, maar ik ontwikkel en ondersteun het in mijn vrije tijd. Je kunt mij bedanken of ondersteunen via een donatie.';
$lang['js']['indexmenuwizard'] = 'Indexmenu Wizard';
$lang['js']['index']           = 'Index';
$lang['js']['options']         = 'Opties';
$lang['js']['navigation']      = 'Navigatie';
$lang['js']['sort']            = 'Sorteren';
$lang['js']['filter']          = 'Filteren';
$lang['js']['performance']     = 'Prestaties';
$lang['js']['namespace']       = 'Namespace';
$lang['js']['nsdepth']         = 'Diepte';
$lang['js']['js']              = 'Boom gerenderd door Javascript, je kunt zelf een thema kiezen';
$lang['js']['theme']           = 'Thema';
$lang['js']['navbar']          = 'De boom opent op de huidige namespace';
$lang['js']['context']         = 'Geeft de boom weer van de huidige wiki namespace context';
$lang['js']['nocookie']        = 'Onthoud niet de open/gesloten knooppunten tijdens het gebruik';
$lang['js']['noscroll']        = 'Voorkom scrollen van de boom als deze niet in de kolombreedte past';
$lang['js']['notoc']           = 'Schakel de inhoudsopgave-voorbeeldfunctie uit';
$lang['js']['tsort']           = 'Op titel';
$lang['js']['dsort']           = 'Op datum';
$lang['js']['msort']           = 'Op metatag';
$lang['js']['nsort']           = 'Sorteer ook de namespaces';
$lang['js']['hsort']           = 'Sorteer startpagina\'s bovenaan';
$lang['js']['rsort']           = 'Keer de sorteervolgorde van pagina\'s om';
$lang['js']['nons']            = 'Geef alleen pagina\'s weer';
$lang['js']['nopg']            = 'Geef alleen namespaces weer';
$lang['js']['max']             = 'Hoeveel niveau\'s er weergegeven worden met AJAX als een knooppunt is geopend. Daarnaast is het aantal subniveau\'s onder dat niveau dat later met AJAX wordt ingeladen, in plaats van meteen bij weergeven van de pagina.';
$lang['js']['maxjs']           = 'Hoeveel niveau\'s worden er weergegeven in de browser als een knooppunt wordt geopend';
$lang['js']['id']              = 'Zelf een cookie-id opgeven voor deze indexmenu';
$lang['js']['insert']          = 'Indexmenu Invoegen';
$lang['js']['metanum']         = 'Metadata: sorteernummer';
$lang['js']['insertmetanum']   = 'Sorteernummer Invoegen';
$lang['js']['page']            = 'Pagina';
$lang['js']['revs']            = 'Revisies';
$lang['js']['tocpreview']      = 'TOC voorbeeld';
$lang['js']['editmode']        = 'Bewerkingsmodus';
$lang['js']['insertdwlink']    = 'Invoegen als DWlink';
$lang['js']['insertdwlinktooltip'] = 'Voeg de link naar deze pagina in in het bewerkingsveld op de positie van de cursor';
$lang['js']['ns']              = 'Namespace';
$lang['js']['search']          = 'Zoeken ...';
$lang['js']['searchtooltip']   = 'Zoeken naar pagina\'s binnen deze namespace';
$lang['js']['create']          = 'Aanmaken';
$lang['js']['more']            = 'Meer';
$lang['js']['headpage']        = 'Hoofdpagina';
$lang['js']['headpagetooltip'] = 'Maak een nieuwe hoofdpagina onder deze pagina';
$lang['js']['startpage']       = 'Startpagina';
$lang['js']['startpagetooltip'] = 'Maak een nieuwe startpagina onder deze pagina';
$lang['js']['custompage']      = 'Gewone pagina';
$lang['js']['custompagetooltip'] = 'Maak een nieuwe pagina onder deze pagina';
$lang['js']['acls']            = 'ACL\'s';
$lang['js']['purgecache']      = 'Wis cache';
$lang['js']['exporthtml']      = 'Exporteer als HTML';
$lang['js']['exporttext']      = 'Exporteer als tekst';
$lang['js']['headpagehere']    = 'Hoofdpagina hier';
$lang['js']['headpageheretooltip'] = 'Maak een nieuwe hoofdpagina binnen deze namespace';
$lang['js']['newpage']         = 'Nieuwe pagina';
$lang['js']['newpagetooltip']  = 'Maak een nieuwe pagina (naam invoeren via popup) in deze namespace';
$lang['js']['newpagehere']     = 'Nieuwe pagina hier';
$lang['js']['insertkeywords']  = 'Voer hier uw zoektermen in voor zoeken binnen deze namespace';
$lang['js']['insertpagename']  = 'Vul de naam in voor de nieuwe pagina';
$lang['js']['edit']            = 'Bewerken';
$lang['js']['loading']         = 'Laden...';
