<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Hideaki SAWADA <chuno@live.jp>
 */
$lang['secedit_name']          = 'テーブル';
$lang['add_table']             = '新規テーブルの挿入';
$lang['js']['toggle_header']   = 'ヘッダー状態の切替';
$lang['js']['align_left']      = '左揃え';
$lang['js']['align_center']    = '中央揃え';
$lang['js']['align_right']     = '右揃え';
$lang['js']['confirmdeleterow'] = '本当に行を削除しますか？';
$lang['js']['confirmdeletecol'] = '本当に列を削除しますか？';
$lang['js']['row_above']       = '上に行を追加する';
$lang['js']['remove_row']      = '行を削除する';
$lang['js']['row_below']       = '下に行を追加する';
$lang['js']['col_left']        = '左に列を追加する';
$lang['js']['remove_col']      = '列を削除する';
$lang['js']['col_right']       = '右列を追加する';
$lang['js']['merge_cells']     = 'セルを結合する';
$lang['js']['unmerge_cells']   = 'セルを分割する';
