<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Jens Hyllegaard <jens.hyllegaard@gmail.com>
 */
$lang['menu']                  = 'Indexmenu værktøjer';
$lang['fetch']                 = 'Vis';
$lang['install']               = 'Installer';
$lang['delete']                = 'Slet';
$lang['check']                 = 'Marker';
$lang['disabled']              = 'Deaktiveret';
$lang['conn_err']              = 'Forbindelsesfejl';
$lang['dir_err']               = 'Kunne ikke oprette midlertidig mappe til at indeholde tema';
$lang['down_err']              = 'Kunne ikke modtage tema';
$lang['zip_err']               = 'ZIP oprettelses eller udpakningsfejl';
$lang['install_ok']            = 'tema installeret korrekt. Det nye tema er til rådighed i værktøjslinjen for redigeringssiden eller med <code>js#theme_name option</code>.';
$lang['install_no']            = 'Forbindelsesfejl. Du kan dog prøve at downloade dit tema manuelt fra <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">dette sted</a>.';
$lang['delete_ok']             = 'Tema blev slettet korrekt.';
$lang['delete_no']             = 'Der opstod en fejl ved sletning af tema';
$lang['upload']                = 'Del';
$lang['checkupdates']          = 'Udvidelsesopdateringer';
$lang['noupdates']             = 'Indexmenu skal ikke opdateres. Du har allerede den nyeste udgave:';
$lang['js']['indexmenuwizard'] = 'Indexmenu guide';
$lang['js']['index']           = 'Index';
$lang['js']['options']         = 'Indstillinger';
$lang['js']['navigation']      = 'Navigation';
$lang['js']['sort']            = 'Sorter';
$lang['js']['filter']          = 'Filtrer';
$lang['js']['performance']     = 'Ydelse';
$lang['js']['namespace']       = 'Navnerum';
$lang['js']['nsdepth']         = 'Dybde';
$lang['js']['tsort']           = 'Efter titel';
$lang['js']['dsort']           = 'Efter dato';
$lang['js']['msort']           = 'Efter metatag';
$lang['js']['nsort']           = 'Sorter også navnerum';
$lang['js']['nons']            = 'Vis kun sider';
$lang['js']['nopg']            = 'Vis kun navnerum';
$lang['js']['insert']          = 'Indsæt indexmenu';
$lang['js']['metanum']         = 'Metanummer for sortering';
$lang['js']['insertmetanum']   = 'Indsæt metanummer';
