<?php

$lang['mail_create_new_error_subject'] = 'gitbacked plugin: FEHLER beim Anlegen eines neuen Git Repositories';
$lang['mail_repo_path_error_subject'] = 'gitbacked plugin: Ein FEHLER mit dem konfigurierten Pfad zum Git Repository ist aufgetreten';
$lang['mail_command_error_subject'] = 'gitbacked plugin: Ein Git Befehl ist FEHLGESCHLAGEN';
$lang['mail_command_success_subject'] = 'gitbacked plugin: Ein Git Befehl wurde ausgeführt';
