<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Thien Hau <thienhau.9a14@gmail.com>
 */
$lang['secedit_name']          = 'Bảng';
$lang['add_table']             = 'Chèn bảng mới';
$lang['js']['toggle_header']   = 'Chuyển trạng thái đầu đề';
$lang['js']['align_left']      = 'Căn trái ô';
$lang['js']['align_center']    = 'Căn giữa ô';
$lang['js']['align_right']     = 'Căn phải ô';
$lang['js']['confirmdeleterow'] = 'Thật sự muốn xóa hàng?';
$lang['js']['confirmdeletecol'] = 'Thật sự muốn xóa cột?';
$lang['js']['row_above']       = 'Thêm hàng vào bên trên';
$lang['js']['remove_row']      = 'Xóa hàng';
$lang['js']['row_below']       = 'Thêm hàng vào bên dưới';
$lang['js']['col_left']        = 'Thêm cột vào bên trái';
$lang['js']['remove_col']      = 'Xóa cột';
$lang['js']['col_right']       = 'Thêm cột vào bên phải';
$lang['js']['merge_cells']     = 'Hợp nhất ô';
$lang['js']['unmerge_cells']   = 'Tách ô';
