<?php
/**
 * Options for the mathjax plugin
 *
 * @author Mark Liffiton <liffiton@gmail.com>
 */

$meta['url'] = array('string');
$meta['config'] = array('');
$meta['configfile'] = array('string');
$meta['asciimath'] = array('onoff');
$meta['mathtags'] = array('string');

