<?php

/**
 * @license    GPL 2 (https://www.gnu.org/licenses/gpl.html)
 *
 * USE : UTF8
 * French language file
 *
 */
$lang['namespaceRoot']         = 'Racine';
$lang['name']                  = 'Nom de la page';
$lang['okbutton']              = 'Créer';
$lang['nooption']              = 'Vous n\'avez pas les droits pour ajouter une page';
