<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Arne Hanssen <arne.hanssen@getmail.no>
 */
$lang['secedit_name']          = 'Tabell';
$lang['add_table']             = 'Sett inn ny tabell';
$lang['js']['toggle_header']   = 'Slå av/på topptekst';
$lang['js']['align_left']      = 'Venstrestill cellen';
$lang['js']['align_center']    = 'Sentrer celle';
$lang['js']['align_right']     = 'Høyrestill celle';
$lang['js']['confirmdeleterow'] = 'Slette rad?';
$lang['js']['confirmdeletecol'] = 'Slette kolonne?';
$lang['js']['row_above']       = 'Legg til rad over';
$lang['js']['remove_row']      = 'Ta bort rad';
$lang['js']['row_below']       = 'Legg til rad under';
$lang['js']['col_left']        = 'Legg til kolonne til venstre';
$lang['js']['remove_col']      = 'Ta bort kolonne';
$lang['js']['col_right']       = 'Legg til kolonne til høyre';
$lang['js']['merge_cells']     = 'Slå sammen celler';
$lang['js']['unmerge_cells']   = 'Splitt celler';
