<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Jakub Duchek <jakduch@jakduch.cz>
 * @author Vojta Olsan <olsh0@seznam.cz>
 */
$lang['menu']                  = 'Indexmenu nástroje';
$lang['fetch']                 = 'Zobrazit';
$lang['install']               = 'Nainstalovat';
$lang['delete']                = 'Smazat';
$lang['check']                 = 'Zkontrolovat';
$lang['no_repos']              = 'Nebyla nastavena adresa úložiště šablon.';
$lang['disabled']              = 'Zakázané';
$lang['conn_err']              = 'Chyba spojení';
$lang['dir_err']               = 'Nezdařilo se vytvoření dočasné složky pro stažení šablony';
$lang['down_err']              = 'Nezdařilo se stažení šablony';
$lang['zip_err']               = 'Nastala chyba při vytváření nebo rozbalování ZIPu';
$lang['delete_ok']             = 'Šablona byla úspěšně smazána';
$lang['delete_no']             = 'Při mazání šablony nastala chyba';
$lang['upload']                = 'Sdílet';
$lang['checkupdates']          = 'Aktualizace pluginů';
$lang['js']['options']         = 'Možnosti';
$lang['js']['navigation']      = 'Navigace';
$lang['js']['sort']            = 'Srovnat';
$lang['js']['filter']          = 'Filtrovat';
$lang['js']['performance']     = 'Výkon';
$lang['js']['nsdepth']         = 'Hloubka';
$lang['js']['theme']           = 'Téma';
$lang['js']['tsort']           = 'Podle názvu';
$lang['js']['dsort']           = 'Podle data';
$lang['js']['msort']           = 'Podle meta tagu';
$lang['js']['nons']            = 'Zobrazit pouze stránky';
$lang['js']['insertmetanum']   = 'Vložit metačíslo';
$lang['js']['page']            = 'Stránka';
$lang['js']['revs']            = 'Revize';
$lang['js']['editmode']        = 'Editační mód';
$lang['js']['insertdwlink']    = 'Vložit jako DWlink';
$lang['js']['search']          = 'Vyhledat ...';
$lang['js']['create']          = 'Vytvořit';
$lang['js']['more']            = 'Více';
$lang['js']['purgecache']      = 'Vyčistit cache';
$lang['js']['exporthtml']      = 'Exportovat jako HTML';
$lang['js']['exporttext']      = 'Exportovat jako text';
$lang['js']['newpage']         = 'Nová stránka...';
$lang['js']['edit']            = 'Upravit';
$lang['js']['loading']         = 'Nahrávání...';
