<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * contextmenu.js 
 *
 * @author Aleksandr Selivanov <alexgearbox@yandex.ru>
 * @author JSBmanD <JSBmanD@gmail.com>
 * @author Вячеслав <gadomskyj@ya.ru>
 * @author Takumo <9206984@mail.ru>
 * @author Andrey Shpak <ashpak@ashpak.ru>
 * @author Chang Zhao <admin@zen-do.ru>
 */
$lang['menu']                  = 'Утилиты Indexmenu';
$lang['fetch']                 = 'Показать';
$lang['install']               = 'Установить';
$lang['delete']                = 'Удалить';
$lang['check']                 = 'Проверить';
$lang['no_repos']              = 'Неверный адрес репозитория тем.';
$lang['disabled']              = 'Отключено';
$lang['conn_err']              = 'Ошибка соединения';
$lang['dir_err']               = 'Ошибка в создании временной папки для принятия темы';
$lang['down_err']              = 'Ошибка принятия темы';
$lang['zip_err']               = 'Ошибка архивации или извлечения';
$lang['install_ok']            = 'Тема успешно установлена. Новые темы доступны в панели инструментов страницы редактирования или с помощью <code>js#theme_name option</code>.';
$lang['install_no']            = 'Ошибка соединения. Тем не менее, вы можете загрузить тему <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">отсюда</a>.';
$lang['delete_ok']             = 'Тема успешно удалена.';
$lang['delete_no']             = 'Ошибка при удалении темы';
$lang['upload']                = 'Поделиться';
$lang['checkupdates']          = 'Обновления плагина';
$lang['noupdates']             = 'Indexmenu не нуждается в обновлении. У вас актуальная версия.';
$lang['infos']                 = 'Можно создать свою тему, следуя инструкциям по адресу <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a>. <br />Также вы сделаете людей счастливее :-), если разместите её в публичном репозитории и создадите кнопку «Поделиться» под темой.';
$lang['showsort']              = 'Номер сортировки: ';
$lang['donation_text']         = 'Разработка плагина не спонсируется сторонними организациями, плагин разрабатывается в моё свободное время. Если вам нравится плагин и/или вы хотите поддержать его разработку, пожалуйста, сделайте пожертвование.';
$lang['js']['indexmenuwizard'] = 'Мастер настройки Indexmenu';
$lang['js']['index']           = 'Индекс';
$lang['js']['options']         = 'Опции';
$lang['js']['navigation']      = 'Навигация';
$lang['js']['sort']            = 'Сортировать';
$lang['js']['filter']          = 'Фильтр';
$lang['js']['performance']     = 'Производительность';
$lang['js']['namespace']       = 'Пространство имён';
$lang['js']['nsdepth']         = 'Вложенность';
$lang['js']['js']              = 'Отображение дерева с использованием Javasript, доступен выбор тем';
$lang['js']['theme']           = 'Тема';
$lang['js']['navbar']          = 'Разворачивать дерево на текущем пространстве имён';
$lang['js']['context']         = 'Показывать дерево текущего пространства имён';
$lang['js']['nocookie']        = 'Не запоминать открытые или закрытые узлы при навигации пользователя';
$lang['js']['noscroll']        = 'Не допускать прокрутку дерева, если оно не помещается в отведённое место';
$lang['js']['notoc']           = 'Заблокировать функцию предпросмотра содержания';
$lang['js']['tsort']           = 'По имени';
$lang['js']['dsort']           = 'По дате';
$lang['js']['msort']           = 'По мета-тегам';
$lang['js']['nsort']           = 'Также сортировать пространства имён';
$lang['js']['hsort']           = 'Сортировать заглавные страницы вверху';
$lang['js']['rsort']           = 'Обратная сортировка страниц';
$lang['js']['nons']            = 'Показывать только страницы';
$lang['js']['nopg']            = 'Показывать только пространства имён';
$lang['js']['max']             = 'Сколько уровней обрабатывать с использованием AJAX, когда узел открыт. Дополнительно указывается, сколько уровней предзагружать вместо 1. ';
$lang['js']['maxjs']           = 'Сколько уровней обрабатывать в браузере клиента, когда узел открыт.';
$lang['js']['id']              = 'Установить cookie id для этого меню';
$lang['js']['insert']          = 'Вставить Indexmenu';
$lang['js']['metanum']         = 'Мета-номер для сортировки';
$lang['js']['insertmetanum']   = 'Вставить мета-номер';
$lang['js']['page']            = 'Страница';
$lang['js']['revs']            = 'История страницы';
$lang['js']['tocpreview']      = 'Предпросмотр';
$lang['js']['editmode']        = 'Режим редактирования';
$lang['js']['insertdwlink']    = 'Вставить как ссылку';
$lang['js']['insertdwlinktooltip'] = 'Вставляет ссылку на страницу на месте курсора в поле редактирования.';
$lang['js']['ns']              = 'Пространство имён';
$lang['js']['search']          = 'Поиск...';
$lang['js']['searchtooltip']   = 'Найти страницы в этом пространстве имён';
$lang['js']['create']          = 'Создать';
$lang['js']['more']            = 'Ещё';
$lang['js']['headpage']        = 'Заглавная страница';
$lang['js']['headpagetooltip'] = 'Создаёт новую заглавную страницу';
$lang['js']['startpage']       = 'Стартовая страница';
$lang['js']['startpagetooltip'] = 'Создаёт новую стартовую страницу';
$lang['js']['custompage']      = 'Новая страница >';
$lang['js']['custompagetooltip'] = 'Создание страницы с произвольным именем';
$lang['js']['acls']            = 'Права доступа';
$lang['js']['purgecache']      = 'Очистить кэш';
$lang['js']['exporthtml']      = 'Сохранить как HTML';
$lang['js']['exporttext']      = 'Сохранить как текст';
$lang['js']['headpagehere']    = 'Заглавная страница';
$lang['js']['headpageheretooltip'] = 'Создать новую заглавную страницу в этом пространстве имён';
$lang['js']['newpage']         = 'Новая страница';
$lang['js']['newpagetooltip']  = 'Создать новую страницу в этом пространстве имён';
$lang['js']['newpagehere']     = 'Создать новую страницу';
$lang['js']['insertkeywords']  = 'Вставьте фразу для поиска в этом контексте имён';
$lang['js']['insertpagename']  = 'Вставьте имя новой страницы';
$lang['js']['edit']            = 'Редактировать';
$lang['js']['loading']         = 'Загрузка...';
