<?php

/**
 * @license    GPL 2 (https://www.gnu.org/licenses/gpl.html)
 *
 * @author João Frade | 100Nome <100nome.portugal@gmail.com>
 */
$lang['secedit_name']          = 'Tabela';
$lang['add_table']             = 'Inserir nova tabela';
$lang['js']['toggle_header']   = 'Alternar estado do cabeçalho';
$lang['js']['align_left']      = 'Alinhar célula à esquerda';
$lang['js']['align_center']    = 'Centrar célula';
$lang['js']['align_right']     = 'Alinhar célula à direita';
$lang['js']['confirmdeleterow'] = 'Eliminar mesmo a linha?';
$lang['js']['confirmdeletecol'] = 'Eliminar mesmo a coluna?';
$lang['js']['row_above']       = 'Adicionar linha acima';
$lang['js']['remove_row']      = 'Remover linha';
$lang['js']['row_below']       = 'Adicionar linha abaixo';
$lang['js']['col_left']        = 'Adicionar coluna ';
$lang['js']['remove_col']      = 'Remover coluna';
$lang['js']['col_right']       = 'Adicionar coluna à direita';
$lang['js']['merge_cells']     = 'Unir células';
$lang['js']['unmerge_cells']   = 'Dividir células';
