<?php
$lang['secedit_name']           = 'Table';
$lang['add_table']              = 'Insert a new table';

$lang['js']['toggle_header']    = 'Toggle header state';
$lang['js']['align_left']       = 'Left-align cell';
$lang['js']['align_center']     = 'Center cell';
$lang['js']['align_right']      = 'Right-align cell';

$lang['js']['confirmdeleterow'] = 'Really delete row?';
$lang['js']['confirmdeletecol'] = 'Really delete column?';

$lang['js']['row_above']        = 'Add row above';
$lang['js']['remove_row']       = 'Remove row';
$lang['js']['row_below']        = 'Add row below';

$lang['js']['col_left']         = 'Add column on the left';
$lang['js']['remove_col']       = 'Remove column';
$lang['js']['col_right']        = 'Add column on the right';

$lang['js']['merge_cells']      = 'Merge cells';
$lang['js']['unmerge_cells']    = 'Split cells';
