<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Domingo Redal <docxml@gmail.com>
 */
$lang['secedit_name']          = 'Tabla';
$lang['add_table']             = 'Insertar una nueva tabla';
$lang['js']['toggle_header']   = 'Alternar el estado del encabezado';
$lang['js']['align_left']      = 'Celda alineada a la izquierda';
$lang['js']['align_center']    = 'Celda centrada';
$lang['js']['align_right']     = 'Celda alineada a la derecha';
$lang['js']['confirmdeleterow'] = '¿Realmente desea borrar la fila?';
$lang['js']['confirmdeletecol'] = '¿Realmente desea borrar la columna?';
$lang['js']['row_above']       = 'Agregar una fila arriba';
$lang['js']['remove_row']      = 'Eliminar fila';
$lang['js']['row_below']       = 'Agregar una fila debajo';
$lang['js']['col_left']        = 'Añadir columna a la izquierda';
$lang['js']['remove_col']      = 'Eliminar columna';
$lang['js']['col_right']       = 'Añadir columna a la derecha';
$lang['js']['merge_cells']     = 'Combinar celdas';
$lang['js']['unmerge_cells']   = 'Dividir celdas';
