<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Aikaterini CATAPODI <extragold1234@hotmail.com>
 * @author Katerina Katapodi <katerinakatapodi@gmail.com>
 */
$lang['menu']                  = 'Χρησιμότητες του Μενού περιεχομένων';
$lang['fetch']                 = 'Προβολή';
$lang['install']               = 'Εγκατάσταση';
$lang['delete']                = 'Απαλοιφή';
$lang['check']                 = 'Έλεγχος ';
$lang['no_repos']              = 'Δεν υπάρχει στοκ ιστοσελίδων';
$lang['disabled']              = 'Απενεργοποιήθηκε';
$lang['conn_err']              = 'Σφάλμα σύνδεσης ';
$lang['dir_err']               = 'Δεν μπόρεσε να δημιουργήσει προσωρινό φάκελλο για τη λήψη θέματος ';
$lang['down_err']              = 'Δεν δέχεται θέμα ';
$lang['zip_err']               = 'Δημιουργία φακέλλου Zip και σφάλμα στο άνοιγμα';
$lang['install_ok']            = 'το θέμα εγκαταστάθηκε με επιτυχία Το καινούργιο θέμα είναι διαθέσιμο στην μπάρα εργαλείων της σελίδας έκδοσης ή με τον κωδικό <code>js#theme_name option</code>.	';
$lang['install_no']            = 'Σφάλμα σύνδεσης Ωστόσο μπορείς να προσπαθήσεις να ανεβάσεις με το χέρι το θέμα σου  <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">here</a>.	';
$lang['delete_ok']             = 'Το θέμα αφαιρέθηκε με επιτυχία';
$lang['delete_no']             = 'Συνέβη σφάλμα κατά την διάρκεια απαλοιφής του θέματος ';
$lang['upload']                = 'Αποστολή';
$lang['checkupdates']          = 'Ενημερώσεις περιεχομένου';
$lang['noupdates']             = 'Το μενού Index δεν χρειάζεται να ενημερωθεί. Έχεις ήδη την τελευταία εκδοχή.  ';
$lang['infos']                 = 'Μπορείτε να δημιουργήσετε το θέμα σας ακολουθώντας τις οδηγίες στο <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a> σελίδα. <br />Έτσι μπορείτε να δώσετε χαρά σε περισσότερους ανθρώπους happy :-) στέλνοντας το στη θέση αποθήκευσης του μενού περιεχομένων,με το κουμπί  "αποστολής" κάτω από το θέμα..	';
$lang['showsort']              = 'Αριθμός κατάταξης του μενού Index : 	';
$lang['donation_text']         = 'To plugin μενού περιεχομένων δεν σπονσοράρεται από κανέναν αλλά το δημιουργώ και το υποστηρίζω δωρεάν στον ελεύθερο χρόνο μου. Αν λάβετε κάτι χάρις σε αυτό ή θελήσετε να συμμετάσχετε στην δημιουργία του, μπορείτε να κάνετε μια δωρεά. 	';
$lang['js']['indexmenuwizard'] = 'Μάγος του Μενού Περιεχομένων';
$lang['js']['index']           = 'Περιεχόμενα ';
$lang['js']['options']         = 'Επιλογές ';
$lang['js']['navigation']      = 'Πλοήγηση ';
$lang['js']['sort']            = 'Κατάταξη ';
$lang['js']['filter']          = 'Φίλτρο';
$lang['js']['performance']     = 'Απόδοση';
$lang['js']['namespace']       = 'Χώρος για τίτλο στο αρχείο';
$lang['js']['nsdepth']         = 'Βάθος ';
$lang['js']['js']              = 'Το δένδρο σχεδιάστηκε με Javasript, μπορείτε να ορίσετε το δικό σας θέμα	';
$lang['js']['theme']           = 'Θέμα';
$lang['js']['navbar']          = 'Το δένδρο ανοίγει στο παρόν αρχείο που χρειάζεται τίτλο';
$lang['js']['context']         = 'Προβάλετε το δένδρο του περιεχομένου αρχείου που χρειάζεται τίτλο';
$lang['js']['nocookie']        = 'Μην θυμάστε τους ανοικτούς/κλειστούς κόμβους της περιήγησης χρήστη';
$lang['js']['noscroll']        = 'Αποφύγετε την κύλιση δένδρου όταν δεν ταιριάζει στο πλάτος του πλαισίου περιεχομένου';
$lang['js']['notoc']           = 'Απενεργοποιήστε την επισκόπηση';
$lang['js']['tsort']           = 'Ανά τίτλο';
$lang['js']['dsort']           = 'Ανά ημερομηνία';
$lang['js']['msort']           = 'Ανά τίτλο meta';
$lang['js']['nsort']           = 'Επιλογή κενά αρχεία για τίτλο';
$lang['js']['hsort']           = 'Επιλέξετε την αρχική σελίδα';
$lang['js']['rsort']           = 'Γυρίζετε σελίδες';
$lang['js']['nons']            = 'Προβολή μόνο σελίδων';
$lang['js']['nopg']            = 'Προβάλετε μόνο αρχεία που δεν έχουν τίτλο';
$lang['js']['max']             = 'Καθορισμός αριθμού επιπέδων με ajax  όταν είναι ανοικτός ένας κόμβος.Επιπλέον επιλέξετε πόσες υποκατηγορίες επιπέδων έχουν ανακτηθεί με την χρήση του AJAX μία φορά. 	';
$lang['js']['maxjs']           = 'Επιλογή αριθμού επιπέδων στον περιηγητή του πελάτη όταν ένας κόμβος είναι ανοικτός';
$lang['js']['id']              = 'Id cookie που προορίζεται για αυτό το μενού περιεχομένων';
$lang['js']['insert']          = 'Εισάγετε μενού περιεχομένου';
$lang['js']['metanum']         = 'Αριθμός Meta που πρέπει να επιλέξετε';
$lang['js']['insertmetanum']   = 'Εισαγωγή αριθμού meta';
$lang['js']['page']            = 'Σελίδα';
$lang['js']['revs']            = 'Επαναλήψεις';
$lang['js']['tocpreview']      = 'Επισκόπηση toc';
$lang['js']['editmode']        = 'Προς έκδοση';
$lang['js']['insertdwlink']    = 'Εισαγωγή ως DWlink';
$lang['js']['insertdwlinktooltip'] = 'Εισάγετε τον σύνδεσμο αυτής της σελίδας στο κουτί έκδοσης στη θέση του κέρσορα';
$lang['js']['ns']              = 'Κενό αρχείο για εισαγωγή τίτλου';
$lang['js']['search']          = 'Αναζήτηση';
$lang['js']['searchtooltip']   = 'Αναζητήστε σελίδες στα πλαίσια αυτής της σειράς ονομάτων';
