<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author lioujheyu <lioujheyu@gmail.com>
 * @author kerry <kerry.eva@gmail.com>
 */
$lang['secedit_name']          = '表格';
$lang['add_table']             = '插入新表格';
$lang['js']['toggle_header']   = '切換標題狀態';
$lang['js']['align_left']      = '置左對齊';
$lang['js']['align_center']    = '置中對齊';
$lang['js']['align_right']     = '置右對齊';
$lang['js']['confirmdeleterow'] = '確認刪除整行？';
$lang['js']['confirmdeletecol'] = '確認刪除整列？';
$lang['js']['row_above']       = '從上方插入整行';
$lang['js']['remove_row']      = '移除整行';
$lang['js']['row_below']       = '從下方插入整行';
$lang['js']['col_left']        = '從左方插入整列';
$lang['js']['remove_col']      = '移除整列';
$lang['js']['col_right']       = '從右方插入整列';
$lang['js']['merge_cells']     = '合並單元格';
$lang['js']['unmerge_cells']   = '拆分單元格';
