<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Robert Bogenschneider <bogi@uea.org>
 */
$lang['secedit_name']          = 'Tabelo';
$lang['add_table']             = 'Enmeti novan tabelon';
$lang['js']['toggle_header']   = '(Mal)ŝalti la titollinion';
$lang['js']['align_left']      = 'Maldekstrigi ĉelon';
$lang['js']['align_center']    = 'Centrigi ĉelon';
$lang['js']['align_right']     = 'Dekstrigi ĉelon';
$lang['js']['confirmdeleterow'] = 'Ĉu vere forigi vicon?';
$lang['js']['confirmdeletecol'] = 'Ĉu vere forigi kolumnon?';
$lang['js']['row_above']       = 'Aldoni vicon supre';
$lang['js']['remove_row']      = 'Forigi vivon';
$lang['js']['row_below']       = 'Aldoni vicon sube';
$lang['js']['col_left']        = 'Aldoni kolumnon maldekstre';
$lang['js']['remove_col']      = 'Forigi kolumnon';
$lang['js']['col_right']       = 'Aldoni kolumnon dekstre';
$lang['js']['colspan_add']     = 'Kunfandi pli da kolumnoj';
$lang['js']['colspan_del']     = 'Kunfandi malpli da kolumnoj';
$lang['js']['rowspan_add']     = 'Kunfandi pli da vicoj';
$lang['js']['rowspan_del']     = 'Kunfandi malpli da vicoj';
