<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Schplurtz le Déboulonné <schplurtz@laposte.net>
 * @author Digitalin <digikatya@yahoo.fr>
 */
$lang['secedit_name']          = 'Tableau';
$lang['add_table']             = 'Insérer un nouveau tableau';
$lang['js']['toggle_header']   = '(dés)activer entêtes';
$lang['js']['align_left']      = 'Alignement à gauche';
$lang['js']['align_center']    = 'Centrage';
$lang['js']['align_right']     = 'Alignement à droite';
$lang['js']['confirmdeleterow'] = 'Vraiment supprimer la ligne ?';
$lang['js']['confirmdeletecol'] = 'Vraiment supprimer la colonne ?';
$lang['js']['row_above']       = 'Ajouter une ligne au dessus';
$lang['js']['remove_row']      = 'Supprimer la ligne';
$lang['js']['row_below']       = 'Ajouter une ligne en dessous';
$lang['js']['col_left']        = 'Ajouter une colonne à gauche';
$lang['js']['remove_col']      = 'Supprimer la colonne';
$lang['js']['col_right']       = 'Ajouter une colonne à droite';
$lang['js']['merge_cells']     = 'Fusionner les cellules';
$lang['js']['unmerge_cells']   = 'Scinder les cellules';
