<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Marina Vladi <deldadam@gmail.com>
 * @author DelD <deldadam@gmail.com>
 */
$lang['menu']                  = 'Indexmenü segédprogamok';
$lang['fetch']                 = 'Megjelenítés';
$lang['install']               = 'Telepítés';
$lang['delete']                = 'Törlés';
$lang['check']                 = 'Ellenőrzés';
$lang['no_repos']              = 'Témagyűjtemény URL-címe nincs megadva';
$lang['disabled']              = 'Letiltva';
$lang['conn_err']              = 'Kapcsolódási hiba';
$lang['dir_err']               = 'Nem tudok létrehozni átmeneti könyvtárat a téma letöltéséhez.';
$lang['down_err']              = 'Nem tudom a témát letölteni.';
$lang['zip_err']               = 'ZIP-létrehozás vagy -kicsomagolási hiba';
$lang['install_ok']            = 'A téma telepítése sikerült. Az új téma elérhető az oldalmódosítás eszközsorában vagy a <code>js#theme_name option</code> kóddal.';
$lang['install_no']            = 'Kapcsolódási hiba. A témánkat megpróbálhatjuk manuálisan is feltölteni <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">innen</a>.';
$lang['delete_ok']             = 'A téma sikeresen törölve.';
$lang['delete_no']             = 'Hiba lépett fel a téma törlése közben.';
$lang['upload']                = 'Megosztás';
$lang['checkupdates']          = 'Bővítmény frissítései';
$lang['noupdates']             = 'Az Indexmenüt nem szükséges frissítenünk. Már a legújabb verzió van telepítve: ';
$lang['infos']                 = 'Készíthetünk saját témát a <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Tématankönyv (Theme Tutorial)</a> oldalon található utasításokat követve.<br />Majd sok embernek örömet is okozhatunk :-), ha beküldjük az Indexmenü gyűjteményébe a téma alatt található "Megosztás" gombbal.';
$lang['showsort']              = 'Indexmenü rendező szám';
$lang['donation_text']         = 'Az Indexmenü-bővítményt nem támogatja senki sem, szabad időmben fejlesztem, és nyújtok támogatást hozzá. Ha ennek köszönhetően Ön ezzel haszonra tesz szert, vagy támogatni szeretné a fejlesztést, kérem, viszonozza ezt adomány formájában.';
$lang['js']['indexmenuwizard'] = 'Indexmenü-varázsló';
$lang['js']['index']           = 'Index';
$lang['js']['options']         = 'Beállítások';
$lang['js']['navigation']      = 'Navigálás';
$lang['js']['sort']            = 'Rendezés';
$lang['js']['filter']          = 'Szűrő';
$lang['js']['performance']     = 'Teljesítmény';
$lang['js']['namespace']       = 'Névtér';
$lang['js']['nsdepth']         = 'Mélység';
$lang['js']['js']              = 'Menü készítése JavaScript-tel; saját témát is definiálhatunk';
$lang['js']['theme']           = 'Téma';
$lang['js']['navbar']          = 'Menü megnyitása az aktuális névtérrel';
$lang['js']['context']         = 'Aktuális wiki-névtérrel összefüggő menü megjelenítése';
$lang['js']['nocookie']        = 'Menüpontok nyitott/zárt állapota nem kerül megjegyzésre';
$lang['js']['noscroll']        = 'Menü görgetésének tiltása, ha nem fér el (szélességben) a konténerelemben';
$lang['js']['notoc']           = 'Tartalom előnézetének tiltása';
$lang['js']['tsort']           = 'Cím szerint';
$lang['js']['dsort']           = 'Dátum szerint';
$lang['js']['msort']           = 'Metacímke szerint';
$lang['js']['nsort']           = 'Rendezés névterek szerint is';
$lang['js']['hsort']           = 'Bevezető oldal rendezéskor fent';
$lang['js']['rsort']           = 'Oldalsorrend megfordítása';
$lang['js']['nons']            = 'Csak az oldalak megjelenítése';
$lang['js']['nopg']            = 'Csak a névterek megjelenítése';
$lang['js']['max']             = 'Hány menüszint kerüljön megjelenítésre AJAX használatával egy almenü megnyitásakor. Másképp: az aktuális menü szintje alatt további hány alszint kerüljön AJAX segítségével letöltésre, ahelyett, hogy az összes egyetlen lépésben töltődjön le.';
$lang['js']['maxjs']           = 'Hány menüszint kerüljön megjelenítésre a kliensböngészőben egy almenü megnyitásakor.';
$lang['js']['id']              = 'Ennek az indexmenünek a saját süti azonosítója (ID-je)';
$lang['js']['insert']          = 'Indexmenü beillesztése';
$lang['js']['metanum']         = 'Metaszám rendezéshez';
$lang['js']['insertmetanum']   = 'Metaszám beillesztése';
$lang['js']['page']            = 'Oldal';
$lang['js']['revs']            = 'Revíziók';
$lang['js']['tocpreview']      = 'Tartalomjegyzék előnézete';
$lang['js']['editmode']        = 'Szerkesztési mód';
$lang['js']['insertdwlink']    = 'Beillesztés DokuWiki-hivatkozásként';
$lang['js']['insertdwlinktooltip'] = 'Oldal hivatkozásának beillesztése a szerkesztő ablakba, a kurzor pozíciójánál';
$lang['js']['ns']              = 'Névtér';
$lang['js']['search']          = 'Keresés...';
$lang['js']['searchtooltip']   = 'Oldalak keresése a névtéren belül';
$lang['js']['create']          = 'Készítés';
$lang['js']['more']            = 'Több';
$lang['js']['headpage']        = 'Főoldal';
$lang['js']['headpagetooltip'] = 'Új főoldal készítése ez alá az oldal alá';
$lang['js']['startpage']       = 'Kezdőlap';
$lang['js']['startpagetooltip'] = 'Új kezdőlap készítése ez alá az oldal alá';
$lang['js']['custompage']      = 'Tetszőleges oldal';
$lang['js']['custompagetooltip'] = 'Új oldal készítése ez alá az oldal alá';
$lang['js']['acls']            = 'ACL (jogosultságok)';
$lang['js']['purgecache']      = 'Átmeneti tár (cache) takarítása';
$lang['js']['exporthtml']      = 'Exportálás HTML-fájlba';
$lang['js']['exporttext']      = 'Exportálás szöveges fájlba';
$lang['js']['headpagehere']    = 'Főoldal ide';
$lang['js']['headpageheretooltip'] = 'Új főoldal készítése a névtérben';
$lang['js']['newpage']         = 'Új oldal';
$lang['js']['newpagetooltip']  = 'Új oldal készítése a névtérben';
$lang['js']['newpagehere']     = 'Új oldal ide';
$lang['js']['insertkeywords']  = 'Kulcsszavak ezen a névtéren belüli kereséshez';
$lang['js']['insertpagename']  = 'Készítendő oldal neve';
$lang['js']['edit']            = 'Szerkesztés';
$lang['js']['loading']         = 'Betöltés...';
