<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Erik Bjørn Pedersen <erik.pedersen@shaw.ca>
 * @author Robert Bogenschneider <bogi@uea.org>
 */
$lang['menu']                  = 'Indeksmenu-ilaro';
$lang['fetch']                 = 'Montri';
$lang['install']               = 'Instali';
$lang['delete']                = 'Forigi';
$lang['check']                 = 'Kontroli';
$lang['no_repos']              = 'Neniu tem-URL konfigurita.';
$lang['disabled']              = 'Blokita';
$lang['conn_err']              = 'Eraro dum konektado';
$lang['dir_err']               = 'Ne eblis krei ricevantan portempan temo-dosierujon';
$lang['down_err']              = 'Ne eblis ricevi la temon';
$lang['zip_err']               = 'Eraro dum pakado aŭ malpakado';
$lang['install_ok']            = 'La temo sukcese instaliĝis. La nova temo haveblas en la ilaro de la modifa paĝo aŭ pere de opcio <code>js#tem_nomo</code>.';
$lang['install_no']            = 'Eraro dum konektado. Vi povus provi alŝuti vian temon permane de <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">tie</a>.';
$lang['delete_ok']             = 'La temo sukcese foriĝis.';
$lang['delete_no']             = 'Eraro okazis dum la forigo de temoj';
$lang['upload']                = 'Distribui';
$lang['checkupdates']          = 'Kromaĵ-aktualigoj';
$lang['noupdates']             = 'La indeksmenuo ne bezonas aktualigon. Vi jam havas la lastan version:';
$lang['infos']                 = 'Vi povas krei vian propran temon, kiam vi sekvas la instrukciojn ĉe la <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Tem-Lernilo</a>. <br />Eblus feliĉigi pliajn homojn :-) sendante ĝin al la publika indeksmenu-arkivo, kun la butono "distribui" sub tiu temo.';
$lang['showsort']              = 'Ordiga numero en la indeksmenuo:';
$lang['donation_text']         = 'La indeksmenu-kromaĵo estas sponsorata de neniu, sed mi evoluigas ĝin senpage dum mia libera tempo. Se vi enspezas ion danke al ĝi aŭ se vi volas subteni ĝian evoluadon, konsideru fari donacon.';
$lang['js']['indexmenuwizard'] = 'Indeksmenu-Asistanto';
$lang['js']['index']           = 'Indekso';
$lang['js']['options']         = 'Agordoj';
$lang['js']['navigation']      = 'Navigado';
$lang['js']['sort']            = 'Ordigi';
$lang['js']['filter']          = 'Filtro';
$lang['js']['performance']     = 'Efikeco';
$lang['js']['namespace']       = 'Nomspaco';
$lang['js']['nsdepth']         = 'Profundeco';
$lang['js']['js']              = 'Arbo kompostita de Javaskripto, vi povas difini vian propran temon';
$lang['js']['theme']           = 'Temo';
$lang['js']['navbar']          = 'La arbo malfermiĝas en la momenta nomspaco';
$lang['js']['context']         = 'Montri la arbon de la momenta nomspaca kunteksto';
$lang['js']['nocookie']        = 'Ne memori (mal)fermajn nodojn dum uzanto-navigado';
$lang['js']['noscroll']        = 'Malebligi trairadon de la arbo kiam ĝi ne kongruas al la larĝeco de ĝia kontenero';
$lang['js']['notoc']           = 'Malebligi la antaŭrigardon de la enhavtabelo';
$lang['js']['tsort']           = 'Per titolo';
$lang['js']['dsort']           = 'Per dato';
$lang['js']['msort']           = 'Per meta-etikedo';
$lang['js']['nsort']           = 'Ordigi ankaŭ nomspacojn';
$lang['js']['hsort']           = 'Ordigi kun ĉefpaĝo ĉekape';
$lang['js']['rsort']           = 'Inversigi la ordigadon de paĝoj';
$lang['js']['nons']            = 'Montri nur paĝojn';
$lang['js']['nopg']            = 'Montri nur nomspacojn';
$lang['js']['max']             = 'Kiom da montrendaj ajax-ebenoj, kiam nodo malfermiĝas. Aldone kiom da subebenoj sub tiu ebeno atingeblas per AJAX en unu fojo';
$lang['js']['maxjs']           = 'Kiom da montrendaj ebenoj en la klienta foliumilo, kiam nodo malfermiĝas';
$lang['js']['id']              = 'Identigilo de memdifinita kekseto por tiu indeksmenuo';
$lang['js']['insert']          = 'Enmeti indeksmenuon';
$lang['js']['metanum']         = 'Meta-numero por ordigado';
$lang['js']['insertmetanum']   = 'Enmeti meta-numeron';
$lang['js']['page']            = 'Paĝo';
$lang['js']['revs']            = 'Revizioj';
$lang['js']['tocpreview']      = 'Antaŭmontro de enhavotabelo';
$lang['js']['editmode']        = 'Redakta reĝimo';
$lang['js']['insertdwlink']    = 'Enmeti kiel DW-ligilon';
$lang['js']['insertdwlinktooltip'] = 'Enmetu la ligilon de ĉi tiu paĝo en la redaktejon ĉe la kursorejo';
$lang['js']['ns']              = 'Nomspaco';
$lang['js']['search']          = 'Serĉi...';
$lang['js']['searchtooltip']   = 'Serĉi paĝojn en tiu ĉi nomspaco';
$lang['js']['create']          = 'Krei';
$lang['js']['more']            = 'Pli';
$lang['js']['headpage']        = 'Paĝokapo';
$lang['js']['headpagetooltip'] = 'Krei novan paĝokapon sub tiu ĉi paĝo';
$lang['js']['startpage']       = 'Komenca paĝo';
$lang['js']['startpagetooltip'] = 'Krei novan komencopaĝon sub tiu ĉi paĝo';
$lang['js']['custompage']      = 'Propra paĝo...';
$lang['js']['custompagetooltip'] = 'Krei novan paĝon (enigi la nomon per ŝprucfenestro) sub tiu ĉi paĝo';
$lang['js']['acls']            = 'Alirkontrollistoj';
$lang['js']['purgecache']      = 'Forigi la kaŝmemoron';
$lang['js']['exporthtml']      = 'Eksporti kiel HTML';
$lang['js']['exporttext']      = 'Eksporti kiel teksto';
$lang['js']['headpagehere']    = 'Paĝokapo estos tie ĉi';
$lang['js']['headpageheretooltip'] = 'Krei novan paĝokapon en tiu ĉi nomspaco';
$lang['js']['newpage']         = 'Nova paĝo...';
$lang['js']['newpagetooltip']  = 'Krei novan paĝon (enigi la nomon per ŝprucfenestro) en tiu ĉi nomspaco';
$lang['js']['newpagehere']     = 'Nova paĝo estos tie ĉi';
$lang['js']['insertkeywords']  = 'Enmeti la ŝlosilvorto(j)n serĉota(j)n en tiu ĉi nomspaco';
$lang['js']['insertpagename']  = 'Enmeti la paĝnomon kreotan';
$lang['js']['edit']            = 'Redakti';
$lang['js']['loading']         = 'Ŝarganta...';