<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author king <b4688756@urhen.com>
 * @author Nobuyuki Fukuyori <self@spumoni.org>
 * @author Hideaki SAWADA <chuno@live.jp>
 */
$lang['menu']                  = 'ツール';
$lang['fetch']                 = '表示';
$lang['install']               = 'インストール';
$lang['delete']                = '削除';
$lang['check']                 = '確認';
$lang['no_repos']              = 'テーマ情報のurlが設定されていません。';
$lang['disabled']              = '無効';
$lang['conn_err']              = '接続エラー';
$lang['dir_err']               = 'テーマを受取るための一時フォルダが作成できません';
$lang['down_err']              = 'テーマを受取ることができません';
$lang['zip_err']               = '圧縮または解凍エラー';
$lang['install_ok']            = 'テーマのインストールが完了しました。新しいテーマは、編集ページのツールバーからか、<code>js#theme_name option</code>と記述することで使用できます。';
$lang['install_no']            = '接続エラー。しかし<a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">ここ</a>から自分のテーマを手動でアップロードできます。';
$lang['delete_ok']             = 'テーマの削除が完了しました。';
$lang['delete_no']             = 'テーマの削除でエラーが発生しました。';
$lang['upload']                = '共有';
$lang['checkupdates']          = 'プラグインの更新';
$lang['noupdates']             = 'Indexmenuをアップデートする必要はありません。既に最新の状態です。';
$lang['infos']                 = '<a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">テーマ学習</a>ページの指示に従って自分のテーマを作成できます。<br />その後、テーマの下の「共有」ボタンで、公共 indexmenu リポジトリに送ってより多くの人を幸せにできます。';
$lang['showsort']              = 'Indexmenuソート番号';
$lang['donation_text']         = 'indexmenuプラグインは誰の後援も受けていませんが、機能を継続するために多くの時間が無料で費やされています。これにより、あなたが何かを得られていたり、純粋に開発を支援したい場合は、開発者に寄付をすることも可能です。';
$lang['js']['indexmenuwizard'] = 'Indexmenuウィザード';
$lang['js']['index']           = 'インデックス';
$lang['js']['options']         = 'オプション';
$lang['js']['navigation']      = 'ナビゲーション';
$lang['js']['sort']            = 'ソート';
$lang['js']['filter']          = 'フィルター';
$lang['js']['performance']     = 'パフォーマンス';
$lang['js']['namespace']       = '名前空間';
$lang['js']['nsdepth']         = '深度';
$lang['js']['js']              = 'レンダリングツリーは、Javasriptによって独自のテーマを定義することができます';
$lang['js']['theme']           = 'テーマ';
$lang['js']['navbar']          = '現在の名前空間でツリーを開きます。';
$lang['js']['context']         = '名前空間からツリーを開始させます';
$lang['js']['nocookie']        = 'ユーザー操作結果のノードの開閉を記憶しません。';
$lang['js']['noscroll']        = 'コンテナの幅に合わない場合、ツリーをスクロールしないようにします。';
$lang['js']['notoc']           = '目次プレビュー機能を無効化します。';
$lang['js']['tsort']           = 'タイトル順';
$lang['js']['dsort']           = '日付順';
$lang['js']['msort']           = 'メタ・タグ順';
$lang['js']['nsort']           = '名前空間もソートする';
$lang['js']['rsort']           = 'ページソートを逆順にする';
$lang['js']['nons']            = 'ページのみ表示';
$lang['js']['nopg']            = '名前空間のみ表示';
$lang['js']['insert']          = 'Indexmenuの挿入';
$lang['js']['metanum']         = 'ソート用の Meta number';
$lang['js']['insertmetanum']   = 'Meta number の追加';
$lang['js']['page']            = 'ページ';
$lang['js']['revs']            = '履歴';
$lang['js']['tocpreview']      = '目次プレビュー';
$lang['js']['editmode']        = '編集モード';
$lang['js']['ns']              = '名前空間';
$lang['js']['search']          = '検索中…';
$lang['js']['searchtooltip']   = 'この名前空間内のページを検索する。';
$lang['js']['create']          = '作成';
$lang['js']['headpage']        = 'ヘッドページ';
$lang['js']['startpage']       = 'スタートページ';
$lang['js']['startpagetooltip'] = 'このページの下に新規スタートページを作成する';
$lang['js']['custompagetooltip'] = 'このページの下に新規ページを作成する';
$lang['js']['purgecache']      = 'キャッシュをクリア';
$lang['js']['exporthtml']      = 'HTML で出力する';
$lang['js']['exporttext']      = 'テキストで出力する';
$lang['js']['headpageheretooltip'] = 'この名前空間内に新規ヘッドページを作成する';
$lang['js']['newpage']         = '新規ページ';
$lang['js']['newpagetooltip']  = 'この名前空間内に新規ページを作成する';
$lang['js']['edit']            = '編集';
