<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Gerrit Uitslag <klapinklapin@gmail.com>
 * @author Klap-in <klapinklapin@gmail.com>
 */
$lang['secedit_name']          = 'Tabel';
$lang['add_table']             = 'Een nieuwe tabel invoegen';
$lang['js']['toggle_header']   = 'Wissel tussen kop en tekst';
$lang['js']['align_left']      = 'Cel links uitlijnen';
$lang['js']['align_center']    = 'Cel gecentreerd uitlijnen';
$lang['js']['align_right']     = 'Cel rechts uitlijnen';
$lang['js']['confirmdeleterow'] = 'Rij echt verwijderen?';
$lang['js']['confirmdeletecol'] = 'Kolom echt verwijderen?';
$lang['js']['row_above']       = 'Rij invoegen hierboven';
$lang['js']['remove_row']      = 'Rij verwijderen';
$lang['js']['row_below']       = 'Rij invoegen hieronder';
$lang['js']['col_left']        = 'Links kolom invoegen';
$lang['js']['remove_col']      = 'Kolom verwijderen';
$lang['js']['col_right']       = 'Rechts kolom invoegen';
$lang['js']['merge_cells']     = 'Cellen samenvoegen';
$lang['js']['unmerge_cells']   = 'Cellen splitsen';
