<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Nima Hamidian <nima.hamidian@gmail.com>
 */
$lang['secedit_name']          = 'جدول';
$lang['add_table']             = 'وارد کردن یک جدول جدید';
$lang['js']['toggle_header']   = 'حالت سرتیتر';
$lang['js']['align_left']      = 'خانه چپ‌چین';
$lang['js']['align_center']    = 'خانه مرکزچین';
$lang['js']['align_right']     = 'خانه راست‌چین';
$lang['js']['confirmdeleterow'] = 'واقعاً سطر حذف شود؟';
$lang['js']['confirmdeletecol'] = 'واقعا ستون حذف شود؟';
$lang['js']['row_above']       = 'افزودن سطر به بالا';
$lang['js']['remove_row']      = 'حذف سطر';
$lang['js']['row_below']       = 'افزودن سطر به پایین';
$lang['js']['col_left']        = 'افزودن ستون به چپ';
$lang['js']['remove_col']      = 'حذف ستون';
$lang['js']['col_right']       = 'افزودن ستون به بالا';
$lang['js']['merge_cells']     = 'تلفیق خانه‌ها';
$lang['js']['unmerge_cells']   = 'جداکردن خانه‌ها';
