<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Jacob Palm <jacobpalmdk@icloud.com>
 * @author Søren Birk <soer9648@eucl.dk>
 */
$lang['secedit_name']          = 'Tabel';
$lang['add_table']             = 'Indsæt en ny tabel';
$lang['js']['toggle_header']   = 'Skift i mellem header-tilstand';
$lang['js']['align_left']      = 'Venstrejustér celle';
$lang['js']['align_center']    = 'Centrer celle';
$lang['js']['align_right']     = 'Højrejustér celle';
$lang['js']['confirmdeleterow'] = 'Slet række?';
$lang['js']['confirmdeletecol'] = 'Slet kolonne?';
$lang['js']['row_above']       = 'Tilføj række herover';
$lang['js']['remove_row']      = 'Fjern række';
$lang['js']['row_below']       = 'Tilføj række herunder';
$lang['js']['col_left']        = 'Tilføj kolonne til venstre';
$lang['js']['remove_col']      = 'Fjern kolonne';
$lang['js']['col_right']       = 'Tilføj kolonne til højre';
$lang['js']['merge_cells']     = 'Sammenflet celler';
$lang['js']['unmerge_cells']   = 'Opdel celler';
