<?php

$lang['export_pdf_button'] = "Uložiť do PDF";
$lang['needtitle']         = "Prosím, zadajte názov.";
$lang['needns']            = "Prosím, zadajte existujúci menný priestor.";
$lang['notexist']          = "Požadovaná stránka neexistuje.";
$lang['empty']             = "Ešte ste nevybrali žiadne stránky.";
$lang['tocheader']         = "Obsah";
$lang['export_ns']         = 'Uložiť menný priestor "%s:" do súboru %s.pdf';
$lang['forbidden']         = "K týmto stránkam nemáte prístup: %s.<br/><br/>Na vytvorenie knihy z dostupných strán použite možnosť 'Preskočiť zakázané stránky'.";
