<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Samuele Tognini <samuele@samuele.netsons.org>
 * @author Edmondo Di Tucci <snarchio@gmail.com>
 * @author lerdt <pro@nicolas-hemard.eu>
 */
$lang['menu']                  = 'Utility di Indexmenu';
$lang['fetch']                 = 'Mostra';
$lang['install']               = 'Installa';
$lang['delete']                = 'Elimina';
$lang['check']                 = 'Controlla';
$lang['no_repos']              = 'Non &egrave stato configurato nessun indirizzo web per il repository dei temi.';
$lang['disabled']              = 'Disabilitato';
$lang['conn_err']              = 'Errore di connessione';
$lang['dir_err']               = 'Impossibile creare la directory temporanea per scaricare il tema';
$lang['down_err']              = 'Impossibile scaricare il tema';
$lang['zip_err']               = 'Errore durante la creazione o scompattazione dell\' archivio';
$lang['install_ok']            = 'tema installato correttamente. Il nuovo tema &egrave accessibile dalla toolbar degli strumenti quando crei una pagina oppure tramite l\' opzione js#nome_tema di indexmenu.';
$lang['install_no']            = 'Errore durante l\' upload. Puoi comunque provare ad inviarmi manualmente il tema da <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">qui</a>.';
$lang['delete_ok']             = 'Rimozione del tema avvenuta con successo';
$lang['delete_no']             = 'Errore durante la rimozione del tema';
$lang['upload']                = 'Condividi';
$lang['checkupdates']          = 'Aggiornamenti plugin';
$lang['noupdates']             = 'Indexmenu non ha bisogno di essere aggiornato. Hai gi&agrave l\'ultima release:';
$lang['infos']                 = 'Puoi creare il tuo tema seguendo le istruzioni nella pagina del <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Tutorial dei temi</a>.<br /> Poi potreste rendere felice pi&ugrave gente :-) inviandolo nel repository pubblico di indexemnu tramite il pulsante "condividi" sotto il tema della lista.';
$lang['showsort']              = 'Ordinamento in indexmenu: ';
$lang['donation_text']         = 'Il plugin indexmenu non &egrave sponsorizzato da nessuno, ma lo sviluppo e supporto gratuitamente nel mio tempo libero. Se guadagni qualcosa grazie a questo plugin o vuoi supportarne lo sviluppo, puoi farlo con una donazione.';
$lang['js']['index']           = 'Indice';
$lang['js']['options']         = 'Opzioni';
$lang['js']['navigation']      = 'Navigazione';
$lang['js']['sort']            = 'Ordine';
$lang['js']['filter']          = 'Filtro';
$lang['js']['performance']     = 'Performance';
$lang['js']['namespace']       = 'Namespace';
$lang['js']['nsdepth']         = 'Profondità';
$lang['js']['js']              = 'Albero renderizzato da Javascript. Puoi definire il tuo tema personalizzato.';
$lang['js']['theme']           = 'Tema';
$lang['js']['navbar']          = 'L\'albero apre il namespace corrente';
$lang['js']['context']         = 'Visualizza l\'albero del contesto del namespace corrente';
$lang['js']['nocookie']        = 'Non ricordare di aprire/chiudere i nodi durante la navigazione utente';
$lang['js']['noscroll']        = 'Previene lo scorrimento dell\'albero quando non rientra nella larghezza del contenitore';
$lang['js']['notoc']           = 'Disabilita l\'anteprima della TOC (Tabella dei contenuti)';
$lang['js']['tsort']           = 'Per titolo';
$lang['js']['dsort']           = 'Per data';
$lang['js']['msort']           = 'Per meta tag';
$lang['js']['nsort']           = 'Ordina anche i namespaces';
$lang['js']['rsort']           = 'Inverti l\'ordine delle pagine';
$lang['js']['nons']            = 'Mostra solo le pagine';
$lang['js']['nopg']            = 'Mostra solo in namespaces';
$lang['js']['maxjs']           = 'Quanti livelli visualizzare nel browser quando un nodo è aperto';
$lang['js']['id']              = 'Cookie ID per questo indexmenu';
$lang['js']['insert']          = 'Inserisci indexmenu';
$lang['js']['metanum']         = 'Meta numero per ordinare';
$lang['js']['insertmetanum']   = 'Inserisci un metanumero';
$lang['js']['page']            = 'Pagina';
$lang['js']['revs']            = 'Modifiche';
$lang['js']['tocpreview']      = 'Indice';
$lang['js']['ns']              = 'Categoria';
$lang['js']['search']          = 'Cerca...';
$lang['js']['searchtooltip']   = 'Cerca in tutte le pagine della categoria';
$lang['js']['create']          = 'Crea';
$lang['js']['acls']            = 'Access Control Lists (ACLs)';
$lang['js']['exporthtml']      = 'Esporta come HTML';
$lang['js']['exporttext']      = 'Esporta come testo';
$lang['js']['newpage']         = 'Nuova pagina...';
$lang['js']['newpagehere']     = 'Crea nuova pagina qui';
$lang['js']['insertkeywords']  = 'Inserisci una o più parole chiavi per cercare in questa categoria';
