<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author qezwan <qezwan@gmail.com>
 */
$lang['menu']                  = 'پێڕستی خزمەتگوزاری';
$lang['fetch']                 = 'نیشاندان';
$lang['install']               = 'دامەزراندن';
$lang['delete']                = 'سڕینەوە';
$lang['check']                 = 'پشکنین';
$lang['no_repos']              = 'هیچ بەستەری بارکراو شێوەبەند نەکراوە.';
$lang['disabled']              = 'ناچالاککراوە';
$lang['conn_err']              = 'هەڵەی پەیوەندی';
$lang['dir_err']               = 'ناتوانێت فۆڵدەری کاتی دروست بکات بۆ وەرگرتنی ڕووکار';
$lang['down_err']              = 'ناتوانێت ڕووکار وەربگرێت';
$lang['zip_err']               = 'هەڵەی دروست کردن یان دەرهێنانی زیپ';
$lang['install_ok']            = 'ڕووکار بە سەرکەوتوویی دامەزرا. ڕووکارە نوێیەکە لە شریتی ئامڕازی لاپەڕەی بژارکردن یان لەگەڵ <code>js#theme_name</code>.';
$lang['install_no']            = 'هەڵەی پەیوەندی. هەرچۆنێک بێت دەتوانیت ڕووکارەکەت بە دەستی باربکەیت لە <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">لێرە</a>.';
$lang['delete_ok']             = 'ڕووکار بە سەرکەوتوویی سڕاوەتەوە.';
$lang['delete_no']             = 'هەڵەیەک ڕوویدا لەکاتی سڕینەوەی ڕووکار';
$lang['upload']                = 'هاوبەش کردن';
$lang['checkupdates']          = 'نوێکردنەوەکانی پێوەکراو';
$lang['noupdates']             = 'پێڕستی پێڕست پێویستی بە نوێکردنەوە نییە. تۆ پێشتر دوا وەشانت هەیە:';
$lang['infos']                 = 'دەتوانیت ڕووکارەکەت دروست بکەیت دوای ڕێنماییەکان لە <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a> page. <br/>دواتر دەتوانیت زیاتر خەڵک دڵخۆش بکەیت :-) ناردنی بۆ کۆگای پێرستی گشتی، لەگەڵ دوگمەی "هاوبەشکردن" لەژێر ئەو ڕووکارە.';
$lang['showsort']              = 'ژمارەی ڕیزکردنی پێڕستی پێڕست: ';
$lang['donation_text']         = 'پێوەکراوەی پێرستمی هیچ کەسێک سپۆنسەری ناکات بەڵام من بە خۆڕایی لە کاتی یەدەگی خۆمدا پەرەی پێ دەدەم و پشتگیری دەکەم . ئەگەر شتێکی سوپاسگوزارت بۆ بەدەست بهێنی یان دەتەوێت پشتگیری لە گەشەی بکەیت، دەتوانیت بیر لە بەخشینێک بکەیت.';
$lang['js']['indexmenuwizard'] = 'ویزاردی پێرست';
$lang['js']['index']           = 'پێڕست';
$lang['js']['options']         = 'هەڵبژاردنەکان';
$lang['js']['navigation']      = 'ڕێنیشاندەر';
$lang['js']['sort']            = 'پۆلێنکردن';
$lang['js']['filter']          = 'فلتەر';
$lang['js']['performance']     = 'ئەدا';
$lang['js']['namespace']       = 'بۆشایی ناو';
$lang['js']['nsdepth']         = 'قوڵایی';
$lang['js']['js']              = 'درەخت لەلایەن Javasript ەوە بە چاپ درا، دەتوانیت ڕووکاری خۆت پێناسە کەیت';
$lang['js']['theme']           = 'ڕووکار';
$lang['js']['navbar']          = 'درەختەکە لە بۆشایی ناوی ئێستا دەکرێتەوە';
$lang['js']['context']         = 'درەختی چوارچێوەی بۆشایی ناوی ویکی ئێستا پیشان بدە';
$lang['js']['nocookie']        = 'بیرت نایەت کە گرێی کراوە/داخراو لە کاتی ڕێدۆزی بەکارهێنەردا';
$lang['js']['noscroll']        = 'ڕێبگرە لە خشاندنی درەختەکە کاتێک لەگەڵ پانتایی دەفرەکە ناجێت';
$lang['js']['notoc']           = 'تایبەتمەندی پێشبینینی toc لە کاربخە';
$lang['js']['tsort']           = 'بە ناونیشان';
$lang['js']['dsort']           = 'بەپێی بەروار';
$lang['js']['msort']           = 'بە تاگی مێتا';
$lang['js']['nsort']           = 'ڕیزکردن هەروەها بۆشایی ناوەکان';
$lang['js']['hsort']           = 'ڕیزکردنی پەڕەی سەرەوە';
$lang['js']['rsort']           = 'پێچەوانەکردنی ڕیزکردنی لاپەڕەکان';
$lang['js']['nons']            = 'تەنها لاپەڕەکان نیشان بدە';
$lang['js']['nopg']            = 'تەنها بۆشایی ناوەکان نیشان بدە';
$lang['js']['max']             = 'چەند ئاست بۆ ئەوەی بە ئەیجاکس بنووسێ کاتێک گرێیەک دەکرێتەوە. زیادە چەند ئاست لە خوارەوەی ئەو ئاستە لەگەڵ ئەیجاکس دەهێنەوە لە جیاتی ئەوەی لە یەک ڕۆیشتندا.';
$lang['js']['maxjs']           = 'چەند ئاست بۆ تێکردن لە خوێنەری کڕیار کاتێک گرێیەک دەکرێتەوە';
$lang['js']['id']              = 'پێناسی کووکی پێناسەکراوی خۆکار بۆ ئەم پێڕستی';
$lang['js']['insert']          = 'مێنوی ئیندێکس دابخە';
$lang['js']['metanum']         = 'ژمارەی مێتا بۆ پۆلێنکردن';
$lang['js']['insertmetanum']   = 'ژمارە دابنێ';
$lang['js']['page']            = 'پەڕە';
$lang['js']['revs']            = 'پێداچوونەوەکان';
$lang['js']['tocpreview']      = 'پێشبینینی Toc';
$lang['js']['editmode']        = 'دەستکاری مۆد';
$lang['js']['insertdwlink']    = 'داخڵکردن وەک DWlink';
$lang['js']['insertdwlinktooltip'] = 'لینکی ئەم لاپەڕەیە لە خانەی بژارکردن دا بنێرە لە شوێنی نیشاندەر';
$lang['js']['ns']              = 'بۆشایی ناو';
$lang['js']['search']          = 'گەڕان ...';
$lang['js']['searchtooltip']   = 'گەڕان بۆ پەڕەکان لەناو ئەم بۆشایی ناوەدا';
$lang['js']['create']          = 'دروستکردن';
$lang['js']['more']            = 'زیاتر';
$lang['js']['headpage']        = 'پەڕەی سەر';
$lang['js']['headpagetooltip'] = 'پەڕەیەکی سەرەکی نوێ دروست بکە لەژێر ئەم لاپەڕەیە';
$lang['js']['startpage']       = 'پەڕەی سەرەتا';
$lang['js']['startpagetooltip'] = 'دروستکردنی لاپەرەیەکی نوێی دەستپێک لەژێر ئەم لاپەڕەیە';
$lang['js']['custompage']      = 'پەڕەی ئاسایی...';
$lang['js']['custompagetooltip'] = 'دروستکردنی لاپەڕەیەکی نوێ (تێکردنی ناو لەڕێی دەرکەوتە) لەژێر ئەم لاپەڕەیە';
$lang['js']['acls']            = 'Acls';
$lang['js']['purgecache']      = 'پاککردنەوە حەشارگە';
$lang['js']['exporthtml']      = 'ناردن وەک HTML';
$lang['js']['exporttext']      = 'ناردن وەک دەق';
$lang['js']['headpagehere']    = 'پەڕەی سەرەکی لێرە';
$lang['js']['headpageheretooltip'] = 'دروستکردنی پەڕەیەکی سەرەکی نوێ لەناو ئەم بۆشایی ناوەدا';
$lang['js']['newpage']         = 'پەڕەی نوێ...';
$lang['js']['newpagetooltip']  = 'لاپەڕەیەکی نوێ دروست بکە (ناوی تێکردن لەڕێی دەرکەوتە) لەناو ئەم بۆشایی ناوەدا';
$lang['js']['newpagehere']     = 'لاپەڕەیەکی نوێ لێرە';
$lang['js']['insertkeywords']  = 'دانانی وشە(ەکان) بۆ گەڕان بۆ ناو ئەم بۆشایی ناوە';
$lang['js']['insertpagename']  = 'ناوی لاپەڕە بکەنێو بۆ دروستکردنی';
$lang['js']['edit']            = 'دەستکاری';
$lang['js']['loading']         = 'بارکردن...';
