/*--------------------------------------------------|
| dTree 2.05 | www.destroydrop.com/javascript/tree/ |
|---------------------------------------------------|
| Copyright (c) 2002-2003 Geir Landro               |
|--------------------------------------------------*/
/*
 a.nodeFdUrl	        Namespace with url link (headpage)	        js
 a.node 	        Namespace without url link        	        js
 a.nodeUrl	        Page	                                        js
 a.nodeSel 	        Last visited page            	                js
 a.navSel 	        Current page            	                js
 a.indexmenu_idx_head	link style of a namespace with url (headpage)	nojs
 a.indexmenu_idx	link style of a namespace without url	        nojs
 */ 

/* dtree properties. No need to change*/

.dtree .thread2{
  font-family: Comic Sans MS,Charter;
}

.dtree .thread2 a.navSel {
  background-color: #000;
  color: #fff
}

.thread2 .indexmenu_rmenuhead {
  background:#000000 url(../../images/close.gif) no-repeat scroll;
  border-bottom: 1px solid #000000;
  color:#ffffff;
}
