<?php

$lang['mail_create_new_error_subject'] = 'gitbacked plugin: The creation of a new Git repo FAILED';
$lang['mail_repo_path_error_subject'] = 'gitbacked plugin: A FAILURE with the Git repo path occurred';
$lang['mail_command_error_subject'] = 'gitbacked plugin: A Git command FAILED';
$lang['mail_command_success_subject'] = 'gitbacked plugin: A Git command was performed';
