<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Yuriy Skalko <yuriy.skalko@gmail.com>
 * @author Aleksandr Selivanov <alexgearbox@yandex.ru>
 * @author artem <blast@ukr.net>
 */
$lang['secedit_name']          = 'Таблица';
$lang['add_table']             = 'Вставить новую таблицу';
$lang['js']['toggle_header']   = 'Заголовок';
$lang['js']['align_left']      = 'По левому краю';
$lang['js']['align_center']    = 'По центру';
$lang['js']['align_right']     = 'По правому краю';
$lang['js']['confirmdeleterow'] = 'Действительно удалить строку?';
$lang['js']['confirmdeletecol'] = 'Действительно удалить столбец?';
$lang['js']['row_above']       = 'Добавить строку выше';
$lang['js']['remove_row']      = 'Удалить строку';
$lang['js']['row_below']       = 'Добавить строку ниже';
$lang['js']['col_left']        = 'Добавить столбец слева';
$lang['js']['remove_col']      = 'Удалить столбец';
$lang['js']['col_right']       = 'Добавить столбец справа';
$lang['js']['merge_cells']     = 'Объединить ячейки';
$lang['js']['unmerge_cells']   = 'Разделить ячейки';
