<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Myeongjin <aranet100@gmail.com>
 */
$lang['secedit_name']          = '표';
$lang['add_table']             = '새 표 넣기';
$lang['js']['toggle_header']   = '머리글 상태 토글';
$lang['js']['align_left']      = '왼쪽 정렬';
$lang['js']['align_center']    = '가운데 정렬';
$lang['js']['align_right']     = '오른쪽 정렬';
$lang['js']['confirmdeleterow'] = '정말 행을 삭제하시겠습니까?';
$lang['js']['confirmdeletecol'] = '정말 열을 삭제하시겠습니까?';
$lang['js']['row_above']       = '위에 행 추가';
$lang['js']['remove_row']      = '행 삭제';
$lang['js']['row_below']       = '아래에 행 추가';
$lang['js']['col_left']        = '왼쪽에 열 추가';
$lang['js']['remove_col']      = '열 삭제';
$lang['js']['col_right']       = '오른쪽에 열 추가';
$lang['js']['merge_cells']     = '셀 병합';
$lang['js']['unmerge_cells']   = '셀 분할';
