<?php return array(
    'root' => array(
        'name' => 'splitbrain/dw2pdf',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => '7d636181fc5bb007c30530a67e7c156f19ded673',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'mpdf/mpdf' => array(
            'pretty_version' => 'v8.0.17',
            'version' => '8.0.17.0',
            'reference' => '5f64118317c8145c0abc606b310aa0a66808398a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/mpdf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mpdf/qrcode' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => '0c09fce8b28707611c3febdd1ca424d40f172184',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/qrcode',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'myclabs/deep-copy' => array(
            'pretty_version' => '1.10.1',
            'version' => '1.10.1.0',
            'reference' => '969b211f9a51aa1f6c01d1d2aef56d3bd91598e5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../myclabs/deep-copy',
            'aliases' => array(),
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.10.1',
            ),
        ),
        'paragonie/random_compat' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.3',
            'version' => '1.1.3.0',
            'reference' => '0f73288fd15629204f9d42b7055f72dacbe811fc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'setasign/fpdi' => array(
            'pretty_version' => 'v2.3.3',
            'version' => '2.3.3.0',
            'reference' => '50c388860a73191e010810ed57dbed795578e867',
            'type' => 'library',
            'install_path' => __DIR__ . '/../setasign/fpdi',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'splitbrain/dw2pdf' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => '7d636181fc5bb007c30530a67e7c156f19ded673',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
