<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Thomas Templin <templin@gnuwhv.de>
 * @author Dana <dannax3@gmx.de>
 */
$lang['menu']                  = 'Indexmenu Werkzeuge';
$lang['fetch']                 = 'Anzeigen';
$lang['install']               = 'Installieren';
$lang['delete']                = 'L&ouml;schen';
$lang['check']                 = '&Uuml;berpr&uuml;fen';
$lang['no_repos']              = 'Keine URL f&uuml;r ein Template Repository vorhanden.';
$lang['disabled']              = 'Deaktiviert';
$lang['conn_err']              = 'Verbindung fehlgeschlagen';
$lang['dir_err']               = 'Fehler bei der Erzeugung eines Verzeichnises. Designvorlage konnte nicht gespeichert werden';
$lang['down_err']              = 'Fehler beim Herunterladen der Designvorlage';
$lang['zip_err']               = 'Fehler beim Erstellen oder Entpacken der Zip-Datei';
$lang['install_ok']            = 'Designvorlage wurde erfolgreich erstellt. Die neue Designvorlage ist in der Werkzeugleiste der Konfigurationsseite oder &uuml;ber die js#theme_name Option verf&uuml;gbar.';
$lang['install_no']            = 'Verbindungsfehler. Du kannst aber versuchen, die Designvorlage <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">hier</a> hochzuladen.';
$lang['delete_ok']             = 'Designvorlage wurde erfolgreich gel&ouml;scht.';
$lang['delete_no']             = 'Beim L&ouml;schen ist ein Fehler aufgetreten';
$lang['upload']                = 'Hochladen';
$lang['checkupdates']          = 'updaten';
$lang['noupdates']             = 'Indexmenu muss nicht upgedated werden. Du hast bereits die aktuellste Version:';
$lang['infos']                 = 'Du kannst eine neue Designvorlage erstellen. Beachte die Anleitung im <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Designvorlagen-Tutorial</a. <br />Wenn Du den "Hochladen" Button unter der Desingvorlage dr&uuml;ckst, kannst Du viele Leute gl&uuml;cklich machen :-) dadurch, dass Du sie im &ouml;ffentlichen Indexmenu-Repository bereitstellst.';
$lang['showsort']              = 'Indexmenu Sortierungsnummer: ';
$lang['donation_text']         = 'Das indexmenu-Plugin wurde vom niemandem unterst&uuml;tzt, aber ich entwickle und unterst&uuml;tze es in meiner Freizeit. Wenn Du dich bedanken oder die Entwicklung unterst&uuml;tzen willst, denke doch bitte &uuml;ber eine Spende nach.';
$lang['js']['indexmenuwizard'] = 'Indexmenu-Wizard';
$lang['js']['index']           = 'Index';
$lang['js']['options']         = 'Optionen';
$lang['js']['navigation']      = 'Navigation';
$lang['js']['sort']            = 'Sortierung';
$lang['js']['filter']          = 'Filter';
$lang['js']['performance']     = 'Performance';
$lang['js']['namespace']       = 'Namensraum';
$lang['js']['nsdepth']         = 'Tiefe';
$lang['js']['js']              = 'Der Baum wird in Javascript gerendert, Sie k&ouml;nnen Ihr eigenes Theme festlegen';
$lang['js']['theme']           = 'Theme';
$lang['js']['navbar']          = 'Der Baum &ouml;ffnet am aktuellen Namensraum';
$lang['js']['context']         = 'Den Baum auf Basis des aktuellen Namensraums anzeigen';
$lang['js']['nocookie']        = 'Den Ge&ouml;ffnet/Geschlossen-Status einzelner Knoten nicht speichern w&auml;hrend der Navigation';
$lang['js']['noscroll']        = 'Das Scrollen des Baums ausschalten, wenn er nicht auf die Seite passt';
$lang['js']['notoc']           = 'Vorschau des Inhaltsverzeichnisses deaktivieren';
$lang['js']['tsort']           = 'Nach Titel';
$lang['js']['dsort']           = 'Nach Datum';
$lang['js']['msort']           = 'Nach Meta-Tag';
$lang['js']['nsort']           = 'Auch die Namensr&auml;ume sortieren';
$lang['js']['hsort']           = 'Startseite oben sortieren';
$lang['js']['rsort']           = 'Sortierung der Seiten umdrehen';
$lang['js']['nons']            = 'Nur Seiten zeigen';
$lang['js']['nopg']            = 'Nur Namensr&auml;ume zeigen';
$lang['js']['max']             = 'Wieviele Ebenen sollen mit Ajax geholt werden, wenn ein Knoten ge&ouml;ffnet wird? Au&szlig;erdem: wieviele Unterebenen unterhalb dieser Ebene sollen mit Ajax geholt werden anstatt w&auml;hrend des Seitenaufbaus?';
$lang['js']['maxjs']           = 'Wieviele Ebenen sollen im Browser statt auf dem Server gerendert werden, wenn ein Knoten ge&ouml;ffnet wird?';
$lang['js']['id']              = 'Benutzerspezifische Cookie-ID f&uml;r dieses indexmenu';
$lang['js']['insert']          = 'Indexmenu einf&uml;gen';
$lang['js']['metanum']         = 'Meta-Nummer zur Sortierung';
$lang['js']['insertmetanum']   = 'Metanummer einf&uuml;gen';
$lang['js']['page']            = 'Seite';
$lang['js']['revs']            = 'Berechtigungen';
$lang['js']['editmode']        = 'Editiermodus';
$lang['js']['insertdwlink']    = 'Einfügen als DWlink';
$lang['js']['ns']              = 'Namensraum';
$lang['js']['search']          = 'Suchen ...';
$lang['js']['searchtooltip']   = 'Suchen von Seiten innerhalb dieses Namensraums';
$lang['js']['create']          = 'Erstellen';
$lang['js']['more']            = 'Mehr';
$lang['js']['startpage']       = 'Startseite';
$lang['js']['startpagetooltip'] = 'Erstellen einer neuen Startseite unter dieser Seite';
$lang['js']['custompage']      = 'Benutzerdefinierte Seite';
$lang['js']['custompagetooltip'] = 'Erstellen einer neuen Seite unter dieser Seite';
$lang['js']['acls']            = 'ACLs';
$lang['js']['purgecache']      = 'Cache löschen';
$lang['js']['exporthtml']      = 'Exportieren als HTML';
$lang['js']['exporttext']      = 'Exportieren als Text';
$lang['js']['newpage']         = 'Neue Seite';
$lang['js']['newpagehere']     = 'Neue Seite hier';
$lang['js']['insertpagename']  = 'Einfügen des Seitennamens zum Erstellen';
$lang['js']['edit']            = 'Bearbeiten';
$lang['js']['loading']         = 'Laden...';
