/* the section edit button  */
.dokuwiki div.editbutton_table {
    margin-top: -1.7em; // this is correct for dokuwiki template, but we adjust it via JS later
    float: none;
    display: none; // we make it visible by JavaScript
    margin-bottom: 1em;

    form div.no button,
    form div.no input.button {
        margin-left: 0.6em;
        padding: 0 0.3em;
        background-image: none;
        border-top: none;
        float: none;

        line-height: 1.8em;
        height: 1.8em;


        border-top-right-radius: 0;
        -moz-border-radius-topright: 0;
        -webkit-border-top-right-radius: 0;

        border-top-left-radius: 0;
        -moz-border-radius-topleft: 0;
        -webkit-border-top-left-radius: 0;

        border-bottom-right-radius: 0.5em;
        -moz-border-radius-bottomright: 0.5em;
        -webkit-border-bottom-right-radius: 0.5em;

        border-bottom-left-radius: 0.5em;
        -moz-border-radius-bottomleft: 0.5em;
        -webkit-border-bottom-left-radius: 0.5em;
    }
}
