<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Martin Růžička <martinr@post.cz>
 * @author Jaroslav Lichtblau <jlichtblau@seznam.cz>
 * @author Vojta Olsan <olsh0@seznam.cz>
 */
$lang['secedit_name']          = 'Tabulka';
$lang['add_table']             = 'Vložit novou tabulku';
$lang['js']['toggle_header']   = 'Přepnout stav hlavičky';
$lang['js']['align_left']      = 'Zarovnat text doleva';
$lang['js']['align_center']    = 'Zarovnat text na střed';
$lang['js']['align_right']     = 'Zarovnat text doprava';
$lang['js']['confirmdeleterow'] = 'Určitě chcete smazat řádek?';
$lang['js']['confirmdeletecol'] = 'Určitě chcete smazat sloupec?';
$lang['js']['row_above']       = 'Přidat řádek nad';
$lang['js']['remove_row']      = 'Odstranit řádek';
$lang['js']['row_below']       = 'Přidat řádek pod';
$lang['js']['col_left']        = 'Přidat sloupec před';
$lang['js']['remove_col']      = 'Odstranit sloupec';
$lang['js']['col_right']       = 'Přidat sloupec za';
$lang['js']['merge_cells']     = 'Sloučit buňky';
$lang['js']['unmerge_cells']   = 'Rozdělit buňky';
