.htContextMenu table tbody {

    tr td {
        padding-left: 0;
        padding-right: 0;

        div.htItemWrapper {
            margin-left: 4px;
            padding-left: 20px;
            background-position: center left;
            background-repeat: no-repeat;

            &.toggle_header {
                background-image: url('../plugins/edittable/images/text_heading.png');
            }

            &.align_left {
                background-image: url('../plugins/edittable/images/a_left.png');
            }

            &.align_center {
                background-image: url('../plugins/edittable/images/a_center.png');
            }

            &.align_right {
                background-image: url('../plugins/edittable/images/a_right.png');
            }

            &.row_above {
                background-image: url('../plugins/edittable/images/row_above.png');
            }

            &.remove_row {
                background-image: url('../plugins/edittable/images/remove_row.png');
            }

            &.row_below {
                background-image: url('../plugins/edittable/images/row_below.png');
            }

            &.col_left {
                background-image: url('../plugins/edittable/images/col_left.png');
            }

            &.remove_col {
                background-image: url('../plugins/edittable/images/remove_col.png');
            }

            &.col_right {
                background-image: url('../plugins/edittable/images/col_right.png');
            }

            &.mergeCells {
                padding-left: 0px;

                & div {
                    padding-left: 20px;
                    background-position: center left;
                    background-repeat: no-repeat;

                    &.merge {
                        background-image: url('../plugins/edittable/images/merge_cells.png');
                    }

                    &.unmerge {
                        background-image: url('../plugins/edittable/images/split_cells.png');
                    }
                }
            }
        }
    }

    td.htSeparator div {
        padding-left: 0;
    }

}
