<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Alan Davies <ben.brynsadler@gmail.com>
 */
$lang['secedit_name']          = 'Tabl';
$lang['add_table']             = 'Mewnosod tabl newydd';
$lang['js']['toggle_header']   = 'Toglo cyflwr y pennyn';
$lang['js']['align_left']      = 'Alinio\'r gell i\'r chwith ';
$lang['js']['align_center']    = 'Alinio\'r gell i\'r canol';
$lang['js']['align_right']     = 'Alinio\'r gell i\'r dde';
$lang['js']['confirmdeleterow'] = 'Ydych chi wir am ddileu\'r rhes?';
$lang['js']['confirmdeletecol'] = 'Ydych chi wir am ddileu\'r golofn?';
$lang['js']['row_above']       = 'Ychwanegu rhes uwchben';
$lang['js']['remove_row']      = 'Tynnu rhes';
$lang['js']['row_below']       = 'Ychwanegu rhes o dan';
$lang['js']['col_left']        = 'Ychwanegu colofn i\'r chwith';
$lang['js']['remove_col']      = 'Tynnu colofn';
$lang['js']['col_right']       = 'Ychwanegu colofn i\'r dde';
$lang['js']['merge_cells']     = 'Uno celloedd';
$lang['js']['unmerge_cells']   = 'Hollti celloedd';
