<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * 
 * @author Davor Turkalj <turki.bsc@gmail.com>
 */
$lang['menu']                  = 'Indeks-meni alati';
$lang['fetch']                 = 'Prikaži';
$lang['install']               = 'Postavi';
$lang['delete']                = 'Obriši';
$lang['check']                 = 'Provjeri';
$lang['no_repos']              = 'Nije podešena URL adresa repozitorija teme.';
$lang['disabled']              = 'Onemogućen';
$lang['conn_err']              = 'Greška povezivanja';
$lang['dir_err']               = 'Ne mogu stvoriti privremenu mapu za prihvat teme';
$lang['down_err']              = 'Ne mogu dohvatiti temu';
$lang['zip_err']               = 'Greška pri pakiranju ili otpakiravanju zip-a';
$lang['install_ok']            = 'tema uspješno postavljena. Nova tema je dostupna u alatnoj traci kod uređivanja ili <code>js#naziv_teme</code> kodom.';
$lang['install_no']            = 'Greška pri povezivanju. Međutim možete pokušati ručno učitati vašu temu <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">ovdje</a>.';
$lang['delete_ok']             = 'Tema uspješno obrisana.';
$lang['delete_no']             = 'Dogodila se greška pri brisanju teme';
$lang['upload']                = 'Dijeli';
$lang['checkupdates']          = 'Dopune dodatka';
$lang['noupdates']             = 'Indexmenu ne treba osvježivati. Zadnja inačica je već postavljena:';
$lang['infos']                 = 'Možete napraviti vlastitu temu slijedeći upute na <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a> stranici.<br /> Vlastitu temu možete poslati na javni indexmenu repozitorij, korištenjem "Dijeli" gumba pored te teme i time ju podijeliti s ostalim korisnicima.';
$lang['showsort']              = 'Indexmenu broj sortiranja:';
$lang['donation_text']         = 'Dodatak Indexmenu nije sponzoriran, već ga razvijam i podržavam u svoje slobodno vrijeme. Ako želite zahvaliti ili podržati razvoj, razmotrite davanje donacije.';
$lang['js']['indexmenuwizard'] = 'Indexmenu čarobnjak';
$lang['js']['index']           = 'Kazalo';
$lang['js']['options']         = 'Opcije';
$lang['js']['navigation']      = 'Navigacija';
$lang['js']['sort']            = 'Poredak';
$lang['js']['filter']          = 'Filtar';
$lang['js']['performance']     = 'Performanse';
$lang['js']['namespace']       = 'Imenski prostor';
$lang['js']['nsdepth']         = 'Dubina';
$lang['js']['js']              = 'Stablo generirano Javascript-om, možete definirati vlastite teme';
$lang['js']['theme']           = 'Tema';
$lang['js']['navbar']          = 'Stablo se otvara na tekućem imenskom prostoru';
$lang['js']['context']         = 'Prikaži stablo trenutnog wiki konteksta imenskog prostora';
$lang['js']['nocookie']        = 'Ne pamti otvorene/zatvorene čvorove tijekom kretanja';
$lang['js']['noscroll']        = 'Spriječi pomicanje stabla kada ono ne stane u širinu okvira';
$lang['js']['notoc']           = 'Onemogući pregled sadržaja stranice';
$lang['js']['tsort']           = 'Po naslovu';
$lang['js']['dsort']           = 'Po datumu';
$lang['js']['msort']           = 'Po meta tagovima';
$lang['js']['nsort']           = 'Također sortiraj imenske prostore';
$lang['js']['hsort']           = 'Sortiraj primarne stranice na vrh';
$lang['js']['rsort']           = 'Obrni poredak stranica';
$lang['js']['nons']            = 'Prikaži samo stranice';
$lang['js']['nopg']            = 'Prikaži samo imenske prostore';
$lang['js']['max']             = 'Koliko nivoa da se prikaže kod otvaranja čvora, kada se koristi AJAX. Dodatno koliko podnivoa da se dohvati s AJAX-om, umjesto jedan za jedan.';
$lang['js']['maxjs']           = 'Koliko nivoa da se prikaže u pregledniku kada je čvor otvoren';
$lang['js']['id']              = 'Pred-definirani id kolačića za ovaj indexmenu.';
$lang['js']['insert']          = 'Ubaci indexmenu';
$lang['js']['metanum']         = 'Meta-broj za sortiranje';
$lang['js']['insertmetanum']   = 'Umetni meta-broj';
$lang['js']['page']            = 'Stranica';
$lang['js']['revs']            = 'Revizije';
$lang['js']['tocpreview']      = 'Pregled sadržaja';
$lang['js']['editmode']        = 'Mod izmjene';
$lang['js']['insertdwlink']    = 'Ubaci kao DW vezu';
$lang['js']['insertdwlinktooltip'] = 'Ubaci link ove stranice u okviru uređivača na poziciji pokazivača';
$lang['js']['ns']              = 'Imenski prostor';
$lang['js']['search']          = 'Pretraži ...';
$lang['js']['searchtooltip']   = 'Pretraga stranica unutar ovog imenskog prostora';
$lang['js']['create']          = 'Kreiraj';
$lang['js']['more']            = 'Više';
$lang['js']['headpage']        = 'Zaglavlje';
$lang['js']['headpagetooltip'] = 'Kreiraj novo zaglavlje unutar ove stranice';
$lang['js']['startpage']       = 'Početna stranica';
$lang['js']['startpagetooltip'] = 'Kreiraj novu početnu stranicu unutar ove stranice';
$lang['js']['custompage']      = 'Korisnička stranica';
$lang['js']['custompagetooltip'] = 'Kreiraj novu korisničku stranicu unutar ove stranice';
$lang['js']['acls']            = 'Acls';
$lang['js']['purgecache']      = 'Očisti međuspremnik';
$lang['js']['exporthtml']      = 'Izvezi kao HTML';
$lang['js']['exporttext']      = 'Izvezi kao tekst';
$lang['js']['headpagehere']    = 'Zaglavlje ovdje';
$lang['js']['headpageheretooltip'] = 'Kreiraj novo zaglavlje unutar ovog imenskog prostora';
$lang['js']['newpage']         = 'Nova stranica';
$lang['js']['newpagetooltip']  = 'Kreiraj novu stranicu unutar ovog imenskog prostora';
$lang['js']['newpagehere']     = 'Novu stranicu ovdje';
$lang['js']['insertkeywords']  = 'Unesi ključne riječi za pretragu unutar ovog imenskog prostora';
$lang['js']['insertpagename']  = 'Unesi ime stranice za kreiranje';
$lang['js']['edit']            = 'Izmjeni';
$lang['js']['loading']         = 'Učitavanje...';
