<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Bartek S <sadupl@gmail.com>
 */
$lang['secedit_name']          = 'Tabela';
$lang['add_table']             = 'Dodaj nową tabelę';
$lang['js']['toggle_header']   = 'Przełącz stan nagłówka';
$lang['js']['align_left']      = 'Wyrównanie do lewej komórki';
$lang['js']['align_center']    = 'Wyśrodkowanie komórki';
$lang['js']['align_right']     = 'Wyrównanie do prawej komórki';
$lang['js']['confirmdeleterow'] = 'Na pewno usunąć wiersz?';
$lang['js']['confirmdeletecol'] = 'Na pewno usunąć kolumnę?';
$lang['js']['row_above']       = 'Dodaj wiersz powyżej';
$lang['js']['remove_row']      = 'Usuń wiersz';
$lang['js']['row_below']       = 'Dodaj wiersz poniżej';
$lang['js']['col_left']        = 'Dodaj kolumnę z lewej';
$lang['js']['remove_col']      = 'Usuń kolumnę';
$lang['js']['col_right']       = 'Dodaj kolumnę z prawej';
$lang['js']['merge_cells']     = 'Połącz komórki';
$lang['js']['unmerge_cells']   = 'Rozdziel komórki';
