<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Tor Härnqvist <tor@harnqvist.se>
 */
$lang['secedit_name']          = 'Tabell';
$lang['add_table']             = 'Infoga en ny tabell';
$lang['js']['toggle_header']   = 'Skifta mellan header-tillstånd';
$lang['js']['align_left']      = 'Vänsterjustera cell';
$lang['js']['align_center']    = 'Mittenjustera cell';
$lang['js']['align_right']     = 'Högerjustera cell';
$lang['js']['confirmdeleterow'] = 'Vill du verkligen ta bort rad?';
$lang['js']['confirmdeletecol'] = 'Vill du verkligen ta bort kolumn?';
$lang['js']['row_above']       = 'Lägg till rad ovanför';
$lang['js']['remove_row']      = 'Ta bort rad';
$lang['js']['row_below']       = 'Lägg till rad nedanför';
$lang['js']['col_left']        = 'Lägg till kolumn till vänster';
$lang['js']['remove_col']      = 'Ta bort kolumn';
$lang['js']['col_right']       = 'Lägg till kolumn till höger';
$lang['js']['merge_cells']     = 'Sammanfoga celler';
$lang['js']['unmerge_cells']   = 'Dela celler';
