<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Samuele Tognini <samuele@samuele.netsons.org>
 * @author Gerrit Uitslag <klapinklapin@gmail.com>
 */
$lang['menu']                  = 'Indexmenu Utilities';
$lang['fetch']                 = 'Show';
$lang['install']               = 'Install';
$lang['delete']                = 'Delete';
$lang['check']                 = 'Check';
$lang['no_repos']              = 'No theme repository url configured.';
$lang['disabled']              = 'Disabled';
$lang['conn_err']              = 'Connection error';
$lang['dir_err']               = 'Unable to create temporary folder to receive theme';
$lang['down_err']              = 'Unable to receive theme';
$lang['zip_err']               = 'Zip creation or extraction error';
$lang['install_ok']            = 'theme installed successfully. The new theme is avaiable in the toolbar of the edit page or with the <code>js#theme_name option</code>.';
$lang['install_no']            = 'Connection error. However you can try to upload manually your theme from <a href="http://samuele.netsons.org/dokuwiki/lib/plugins/indexmenu/upload/">here</a>.';
$lang['delete_ok']             = 'Theme is successfully deleted.';
$lang['delete_no']             = 'An error occurred during theme deleting';
$lang['upload']                = 'Share';
$lang['checkupdates']          = 'Plugin updates';
$lang['noupdates']             = 'Indexmenu does not need to be update. You have already the last release:';
$lang['infos']                 = 'You can create your theme following the instructions at the <a href="https://www.dokuwiki.org/plugin:indexmenu#theme_tutorial">Theme Tutorial</a> page. <br />Then you could make more people happy :-) sending it to the public indexmenu repository, with the "share" button under that theme.';
$lang['showsort']              = 'Indexmenu sort number: ';
$lang['donation_text']         = 'The indexmenu plugin is not sponsored by anyone but i develop and support it for free during my spare time. If you gain something thanks to it or you want to support its development, you can consider to make a donation.';
$lang['js']['indexmenuwizard'] = 'Indexmenu Wizard';
$lang['js']['index']           = 'Index';
$lang['js']['options']         = 'Options';
$lang['js']['navigation']      = 'Navigation';
$lang['js']['sort']            = 'Sort';
$lang['js']['filter']          = 'Filter';
$lang['js']['performance']     = 'Performance';

$lang['js']['namespace']       = 'Namespace';
$lang['js']['nsdepth']         = 'Depth';
$lang['js']['js']              = 'Tree rendered by Javasript, you can define your own theme';
$lang['js']['theme']           = 'Theme';
$lang['js']['navbar']          = 'The tree opens at the current namespace';
$lang['js']['context']         = 'Display the tree of the current wiki namespace context';
$lang['js']['nocookie']        = 'Don\'t remember open/closed nodes during user navigation';
$lang['js']['noscroll']        = 'Prevent to scrolling the tree when it does not fit its container width';
$lang['js']['notoc']           = 'Disable the toc preview feature';
$lang['js']['tsort']           = 'By title';
$lang['js']['dsort']           = 'By date';
$lang['js']['msort']           = 'By meta tag';
$lang['js']['nsort']           = 'Sort also namespaces';
$lang['js']['hsort']           = 'Sort headpage up';
$lang['js']['rsort']           = 'Reverse the sorting of pages';
$lang['js']['nons']            = 'Show only pages';
$lang['js']['nopg']            = 'Show only namespaces';
$lang['js']['max']             = 'How many levels to render with ajax when a node is opened. Additional are how many sublevels below that level are retrieved with AJAX instead of in one go. ';
$lang['js']['maxjs']           = 'How many levels to render in the client browser when a node is opened';
$lang['js']['id']              = 'Self defined cookie id for this indexmenu';
$lang['js']['insert']          = 'Insert indexmenu';
$lang['js']['metanum']         = 'Meta number for sorting';
$lang['js']['insertmetanum']   = 'Insert metanumber';

/* contextmenu.js */
$lang['js']['page']            = 'Page';
$lang['js']['revs']            = 'Revisions';
$lang['js']['tocpreview']      = 'Toc preview';
$lang['js']['editmode']        = 'Edit mode';
$lang['js']['insertdwlink']    = 'Insert as DWlink';
$lang['js']['insertdwlinktooltip'] = 'Insert the link of this page in the edit box at cursor position';
$lang['js']['ns']              = 'Namespace';
$lang['js']['search']          = 'Search ...';
$lang['js']['searchtooltip']   = 'Search for pages within this namespace';
$lang['js']['create']          = 'Create';
$lang['js']['more']            = 'More';
$lang['js']['headpage']        = 'Headpage';
$lang['js']['headpagetooltip'] = 'Create a new headpage under this page';
$lang['js']['startpage']       = 'Start page';
$lang['js']['startpagetooltip'] = 'Create a new start page under this page';
$lang['js']['custompage']      = 'Custom page...';
$lang['js']['custompagetooltip'] = 'Create a new page (input the name via popup) under this page';
$lang['js']['acls']            = 'Acls';
$lang['js']['purgecache']      = 'Purge cache';
$lang['js']['exporthtml']      = 'Export as HTML';
$lang['js']['exporttext']      = 'Export as text';
$lang['js']['headpagehere']    = 'Headpage here';
$lang['js']['headpageheretooltip'] = 'Create a new headpage inside this namespace';
$lang['js']['newpage']         = 'New page...';
$lang['js']['newpagetooltip']  = 'Create a new page (input the name via popup) inside this namespace';
$lang['js']['newpagehere']     = 'New page here';
$lang['js']['insertkeywords']  = 'Insert keyword(s) to search for within this namespace';
$lang['js']['insertpagename']  = 'Insert the pagename to create';
$lang['js']['edit']            = 'Edit';
$lang['js']['loading']         = 'Loading...';
