<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 */
$lang['secedit_name']          = 'Tabelle';
$lang['add_table']             = 'Neue Tabelle einfügen';
$lang['js']['toggle_header']   = 'Kopfzeile an/aus';
$lang['js']['align_left']      = 'Links ausrichten';
$lang['js']['align_center']    = 'Mittig ausrichten';
$lang['js']['align_right']     = 'Rechts ausrichten';
$lang['js']['confirmdeleterow'] = 'Zeile wirklich löschen?';
$lang['js']['confirmdeletecol'] = 'Spalte wirklich löschen?';
$lang['js']['row_above']       = 'Zeile darüber einfügen';
$lang['js']['remove_row']      = 'Zeile entfernen';
$lang['js']['row_below']       = 'Zeile darunter einfügen';
$lang['js']['col_left']        = 'Spalte links hinzufügen';
$lang['js']['remove_col']      = 'Spalte entfernen';
$lang['js']['col_right']       = 'Spalte rechts hinzufügen';
$lang['js']['merge_cells']     = 'Zellen verbinden';
$lang['js']['unmerge_cells']   = 'Zellen trennen';
